# NOTE: This stub is incomplete - only contains some global functions

import sys
from typing import Any, Dict, Optional

def run(statement: str, globals: Optional[Dict[str, Any]] = ...,
        locals: Optional[Dict[str, Any]] = ...) -> None: ...
def runeval(expression: str, globals: Optional[Dict[str, Any]] = ...,
            locals: Optional[Dict[str, Any]] = ...) -> Any: ...
def runctx(statement: str, globals: Dict[str, Any], locals: Dict[str, Any]) -> None: ...
def runcall(*args: Any, **kwds: Any) -> Any: ...

if sys.version_info >= (3, 7):
    def set_trace(*, header: Optional[str] = ...) -> None: ...
else:
    def set_trace() -> None: ...

def post_mortem(t: Optional[Any] = ...) -> None: ...
def pm() -> None: ...



# Stubs for uu (Python 2 and 3)
import sys
from typing import BinaryIO, Union, Optional, Text

_File = Union[Text, BinaryIO]

class Error(Exception): ...

if sys.version_info >= (3, 7):
    def encode(in_file: _File, out_file: _File, name: Optional[str] = ..., mode: Optional[int] = ..., backtick: bool = ...) -> None: ...
else:
    def encode(in_file: _File, out_file: _File, name: Optional[str] = ..., mode: Optional[int] = ...) -> None: ...
def decode(in_file: _File, out_file: Optional[_File] = ..., mode: Optional[int] = ..., quiet: int = ...) -> None: ...



# NB: third_party/2/enum.pyi and stdlib/3.4/enum.pyi must remain consistent!
import sys
from typing import Any, Iterator, List, Mapping, Type, TypeVar, Union
from abc import ABCMeta

_T = TypeVar('_T')
_S = TypeVar('_S', bound=Type[Enum])

# Note: EnumMeta actually subclasses type directly, not ABCMeta.
# This is a temporary workaround to allow multiple creation of enums with builtins
# such as str as mixins, which due to the handling of ABCs of builtin types, cause
# spurious inconsistent metaclass structure. See #1595.
# Structurally: Iterable[T], Reversible[T], Container[T] where T is the enum itself
class EnumMeta(ABCMeta):
    def __iter__(self: Type[_T]) -> Iterator[_T]: ...
    def __reversed__(self: Type[_T]) -> Iterator[_T]: ...
    def __contains__(self: Type[_T], member: object) -> bool: ...
    def __getitem__(self: Type[_T], name: str) -> _T: ...
    @property
    def __members__(self: Type[_T]) -> Mapping[str, _T]: ...
    def __len__(self) -> int: ...

class Enum(metaclass=EnumMeta):
    def __new__(cls: Type[_T], value: object) -> _T: ...
    def __repr__(self) -> str: ...
    def __str__(self) -> str: ...
    def __dir__(self) -> List[str]: ...
    def __format__(self, format_spec: str) -> str: ...
    def __hash__(self) -> Any: ...
    def __reduce_ex__(self, proto: object) -> Any: ...

    name = ...  # type: str
    value = ...  # type: Any

class IntEnum(int, Enum):
    value = ...  # type: int

def unique(enumeration: _S) -> _S: ...

if sys.version_info >= (3, 6):
    _auto_null = ...  # type: Any

    # subclassing IntFlag so it picks up all implemented base functions, best modeling behavior of enum.auto()
    class auto(IntFlag):
        value = ...  # type: Any

    class Flag(Enum):
        def __contains__(self: _T, other: _T) -> bool: ...
        def __repr__(self) -> str: ...
        def __str__(self) -> str: ...
        def __bool__(self) -> bool: ...
        def __or__(self: _T, other: _T) -> _T: ...
        def __and__(self: _T, other: _T) -> _T: ...
        def __xor__(self: _T, other: _T) -> _T: ...
        def __invert__(self: _T) -> _T: ...

    # The `type: ignore` comment is needed because mypy considers the type
    # signatures of several methods defined in int and Flag to be incompatible.
    class IntFlag(int, Flag):  # type: ignore
        def __or__(self: _T, other: Union[int, _T]) -> _T: ...
        def __and__(self: _T, other: Union[int, _T]) -> _T: ...
        def __xor__(self: _T, other: Union[int, _T]) -> _T: ...
        __ror__ = __or__
        __rand__ = __and__
        __rxor__ = __xor__



"""Stub file for the '_json' module."""
# This is an autogenerated file. It serves as a starting point
# for a more precise manual annotation of this module.
# Feel free to edit the source below, but remove this header when you do.

from typing import Any, List, Tuple, Dict, Generic

def encode_basestring_ascii(*args, **kwargs) -> str:
    raise TypeError()

def scanstring(a, b, *args, **kwargs) -> tuple:
    raise TypeError()


class Encoder(object): ...

class Scanner(object): ...



# Stubs for OpenSSL.crypto (Python 2)

from typing import Any, Callable, Iterable, List, Optional, Set, Text, Tuple, Union

from cryptography.hazmat.primitives.asymmetric import dsa, rsa
from datetime import datetime

FILETYPE_PEM = ...  # type: int
FILETYPE_ASN1 = ...  # type: int
FILETYPE_TEXT = ...  # type: int
TYPE_RSA = ...  # type: int
TYPE_DSA = ...  # type: int

class Error(Exception): ...

_Key = Union[rsa.RSAPublicKey, rsa.RSAPrivateKey, dsa.DSAPublicKey, dsa.DSAPrivateKey]

class PKey:
    def __init__(self) -> None: ...
    def to_cryptography_key(self) -> _Key: ...
    @classmethod
    def from_cryptography_key(cls, crypto_key: _Key): ...
    def generate_key(self, type: int, bits: int) -> None: ...
    def check(self) -> bool: ...
    def type(self) -> int: ...
    def bits(self) -> int: ...

class _EllipticCurve:
    name = ...  # type: Text

def get_elliptic_curves() -> Set[_EllipticCurve]: ...
def get_elliptic_curve(name: str) -> _EllipticCurve: ...

class X509Name:
    def __init__(self, name: X509Name) -> None: ...
    countryName = ...  # type: Union[str, unicode]
    stateOrProvinceName = ...  # type: Union[str, unicode]
    localityName = ...  # type: Union[str, unicode]
    organizationName = ...  # type: Union[str, unicode]
    organizationalUnitName = ...  # type: Union[str, unicode]
    commonName = ...  # type: Union[str, unicode]
    emailAddress = ...  # type: Union[str, unicode]
    C = ...  # type: Union[str, unicode]
    ST = ...  # type: Union[str, unicode]
    L = ...  # type: Union[str, unicode]
    O = ...  # type: Union[str, unicode]
    OU = ...  # type: Union[str, unicode]
    CN = ...  # type: Union[str, unicode]
    def hash(self) -> int: ...
    def der(self) -> bytes: ...
    def get_components(self) -> List[Tuple[str, str]]: ...

class X509Extension:
    def __init__(self, type_name: bytes, critical: bool, value: bytes, subject: Optional[X509] = ...,
                 issuer: Optional[X509] = ...) -> None: ...
    def get_critical(self) -> bool: ...
    def get_short_name(self) -> str: ...
    def get_data(self) -> str: ...

class X509Req:
    def __init__(self) -> None: ...
    def set_pubkey(self, pkey: PKey) -> None: ...
    def get_pubkey(self) -> PKey: ...
    def set_version(self, version: int) -> None: ...
    def get_version(self) -> int: ...
    def get_subject(self) -> X509Name: ...
    def add_extensions(self, extensions: Iterable[X509Extension]) -> None: ...
    def get_extensions(self) -> List[X509Extension]: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def verify(self, pkey: PKey) -> bool: ...

class X509:
    def __init__(self) -> None: ...
    def set_version(self, version: int) -> None: ...
    def get_version(self) -> int: ...
    def get_pubkey(self) -> PKey: ...
    def set_pubkey(self, pkey: PKey) -> None: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def get_signature_algorithm(self) -> str: ...
    def digest(self, digest_name: str) -> str: ...
    def subject_name_hash(self) -> str: ...
    def set_serial_number(self, serial: int) -> None: ...
    def get_serial_number(self) -> int: ...
    def gmtime_adj_notAfter(self, amount: int) -> None: ...
    def gmtime_adj_notBefore(self, amount: int) -> None: ...
    def has_expired(self) -> bool: ...
    def get_notBefore(self) -> str: ...
    def set_notBefore(self, when: str) -> None: ...
    def get_notAfter(self) -> str: ...
    def set_notAfter(self, when: str) -> None: ...
    def get_issuer(self) -> X509Name: ...
    def set_issuer(self, issuer: X509Name) -> None: ...
    def get_subject(self) -> X509Name: ...
    def set_subject(self, subject: X509Name) -> None: ...
    def get_extension_count(self) -> int: ...
    def add_extensions(self, extensions: Iterable[X509Extension]) -> None: ...
    def get_extension(self, index: int) -> X509Extension: ...

class X509StoreFlags:
    CRL_CHECK = ...  # type: int
    CRL_CHECK_ALL = ...  # type: int
    IGNORE_CRITICAL = ...  # type: int
    X509_STRICT = ...  # type: int
    ALLOW_PROXY_CERTS = ...  # type: int
    POLICY_CHECK = ...  # type: int
    EXPLICIT_POLICY = ...  # type: int
    INHIBIT_MAP = ...  # type: int
    NOTIFY_POLICY = ...  # type: int
    CHECK_SS_SIGNATURE = ...  # type: int
    CB_ISSUER_CHECK = ...  # type: int

class X509Store:
    def __init__(self) -> None: ...
    def add_cert(self, cert: X509) -> None: ...
    def add_crl(self, crl: CRL) -> None: ...
    def set_flags(self, flags: int) -> None: ...
    def set_time(self, vfy_time: datetime) -> None: ...

class X509StoreContextError(Exception):
    certificate = ...  # type: X509
    def __init__(self, message: str, certificate: X509) -> None: ...

class X509StoreContext:
    def __init__(self, store: X509Store, certificate: X509) -> None: ...
    def set_store(self, store: X509Store) -> None: ...
    def verify_certificate(self) -> None: ...

def load_certificate(type: int, buffer: Union[str, unicode]) -> X509: ...
def dump_certificate(type: int, cert: X509) -> bytes: ...
def dump_publickey(type: int, pkey: PKey) -> bytes: ...
def dump_privatekey(type: int, pkey: PKey, cipher: Optional[str] = ...,
                    passphrase: Optional[Union[str, Callable[[int], int]]] = ...) -> bytes: ...

class Revoked:
    def __init__(self) -> None: ...
    def set_serial(self, hex_str: str) -> None: ...
    def get_serial(self) -> str: ...
    def set_reason(self, reason: str) -> None: ...
    def get_reason(self) -> str: ...
    def all_reasons(self) -> List[str]: ...
    def set_rev_date(self, when: str) -> None: ...
    def get_rev_date(self) -> str: ...

class CRL:
    def __init__(self) -> None: ...
    def get_revoked(self) -> Tuple[Revoked, ...]: ...
    def add_revoked(self, revoked: Revoked) -> None: ...
    def get_issuer(self) -> X509Name: ...
    def set_version(self, version: int) -> None: ...
    def set_lastUpdate(self, when: str) -> None: ...
    def set_nextUpdate(self, when: str) -> None: ...
    def sign(self, issuer_cert: X509, issuer_key: PKey, digest: str) -> None: ...
    def export(self, cert: X509, key: PKey, type: int = ..., days: int = ..., digest: str = ...) -> bytes: ...

class PKCS7:
    def type_is_signed(self) -> bool: ...
    def type_is_enveloped(self) -> bool: ...
    def type_is_signedAndEnveloped(self) -> bool: ...
    def type_is_data(self) -> bool: ...
    def get_type_name(self) -> str: ...

class PKCS12:
    def __init__(self) -> None: ...
    def get_certificate(self) -> X509: ...
    def set_certificate(self, cert: X509) -> None: ...
    def get_privatekey(self) -> PKey: ...
    def set_privatekey(self, pkey: PKey) -> None: ...
    def get_ca_certificates(self) -> Tuple[X509, ...]: ...
    def set_ca_certificates(self, cacerts: Iterable[X509]) -> None: ...
    def set_friendlyname(self, name: bytes) -> None: ...
    def get_friendlyname(self) -> bytes: ...
    def export(self, passphrase: Optional[str] = ..., iter: int = ..., maciter: int = ...): ...

class NetscapeSPKI:
    def __init__(self) -> None: ...
    def sign(self, pkey: PKey, digest: str) -> None: ...
    def verify(self, key: PKey) -> bool: ...
    def b64_encode(self) -> str: ...
    def get_pubkey(self) -> PKey: ...
    def set_pubkey(self, pkey: PKey) -> None: ...

def load_publickey(type: int, buffer: Union[str, unicode]) -> PKey: ...
def load_privatekey(type: int, buffer: bytes, passphrase: Optional[Union[str, Callable[[int], int]]] = ...): ...
def dump_certificate_request(type: int, req: X509Req): ...
def load_certificate_request(type, buffer: Union[str, unicode]) -> X509Req: ...
def sign(pkey: PKey, data: Union[str, unicode], digest: str) -> bytes: ...
def verify(cert: X509, signature: bytes, data: Union[str, unicode], digest: str) -> None: ...
def dump_crl(type: int, crl: CRL) -> bytes: ...
def load_crl(type: int, buffer: Union[str, unicode]) -> CRL: ...
def load_pkcs7_data(type: int, buffer: Union[str, unicode]) -> PKCS7: ...
def load_pkcs12(buffer: Union[str, unicode], passphrase: Optional[Union[str, Callable[[int], int]]] = ...) -> PKCS12: ...



"""Utilities for type argument inference."""

from typing import List, Optional, Sequence

from mypy.constraints import infer_constraints, infer_constraints_for_callable
from mypy.types import Type, TypeVarId, CallableType
from mypy.solve import solve_constraints
from mypy.constraints import SUBTYPE_OF


def infer_function_type_arguments(callee_type: CallableType,
                                  arg_types: Sequence[Optional[Type]],
                                  arg_kinds: List[int],
                                  formal_to_actual: List[List[int]],
                                  strict: bool = True) -> List[Optional[Type]]:
    """Infer the type arguments of a generic function.

    Return an array of lower bound types for the type variables -1 (at
    index 0), -2 (at index 1), etc. A lower bound is None if a value
    could not be inferred.

    Arguments:
      callee_type: the target generic function
      arg_types: argument types at the call site (each optional; if None,
                 we are not considering this argument in the current pass)
      arg_kinds: nodes.ARG_* values for arg_types
      formal_to_actual: mapping from formal to actual variable indices
    """
    # Infer constraints.
    constraints = infer_constraints_for_callable(
        callee_type, arg_types, arg_kinds, formal_to_actual)

    # Solve constraints.
    type_vars = callee_type.type_var_ids()
    return solve_constraints(type_vars, constraints, strict)


def infer_type_arguments(type_var_ids: List[TypeVarId],
                         template: Type, actual: Type) -> List[Optional[Type]]:
    # Like infer_function_type_arguments, but only match a single type
    # against a generic type.
    constraints = infer_constraints(template, actual, SUBTYPE_OF)
    return solve_constraints(type_var_ids, constraints)



from google.protobuf.internal.containers import (
    RepeatedCompositeFieldContainer,
    RepeatedScalarFieldContainer,
)
from google.protobuf.message import (
    Message,
)
from typing import (
    Iterable,
    List,
    Optional,
    Text,
    Tuple,
    cast,
)


class FileDescriptorSet(Message):

    @property
    def file(self) -> RepeatedCompositeFieldContainer[FileDescriptorProto]: ...

    def __init__(self,
                 file: Optional[Iterable[FileDescriptorProto]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FileDescriptorSet: ...


class FileDescriptorProto(Message):
    name = ...  # type: Text
    package = ...  # type: Text
    dependency = ...  # type: RepeatedScalarFieldContainer[Text]
    public_dependency = ...  # type: RepeatedScalarFieldContainer[int]
    weak_dependency = ...  # type: RepeatedScalarFieldContainer[int]
    syntax = ...  # type: Text

    @property
    def message_type(
        self) -> RepeatedCompositeFieldContainer[DescriptorProto]: ...

    @property
    def enum_type(
        self) -> RepeatedCompositeFieldContainer[EnumDescriptorProto]: ...

    @property
    def service(
        self) -> RepeatedCompositeFieldContainer[ServiceDescriptorProto]: ...

    @property
    def extension(
        self) -> RepeatedCompositeFieldContainer[FieldDescriptorProto]: ...

    @property
    def options(self) -> FileOptions: ...

    @property
    def source_code_info(self) -> SourceCodeInfo: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 package: Optional[Text] = ...,
                 dependency: Optional[Iterable[Text]] = ...,
                 public_dependency: Optional[Iterable[int]] = ...,
                 weak_dependency: Optional[Iterable[int]] = ...,
                 message_type: Optional[Iterable[DescriptorProto]] = ...,
                 enum_type: Optional[Iterable[EnumDescriptorProto]] = ...,
                 service: Optional[Iterable[ServiceDescriptorProto]] = ...,
                 extension: Optional[Iterable[FieldDescriptorProto]] = ...,
                 options: Optional[FileOptions] = ...,
                 source_code_info: Optional[SourceCodeInfo] = ...,
                 syntax: Optional[Text] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FileDescriptorProto: ...


class DescriptorProto(Message):

    class ExtensionRange(Message):
        start = ...  # type: int
        end = ...  # type: int

        @property
        def options(self) -> ExtensionRangeOptions: ...

        def __init__(self,
                     start: Optional[int] = ...,
                     end: Optional[int] = ...,
                     options: Optional[ExtensionRangeOptions] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: bytes) -> DescriptorProto.ExtensionRange: ...

    class ReservedRange(Message):
        start = ...  # type: int
        end = ...  # type: int

        def __init__(self,
                     start: Optional[int] = ...,
                     end: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: bytes) -> DescriptorProto.ReservedRange: ...
    name = ...  # type: Text
    reserved_name = ...  # type: RepeatedScalarFieldContainer[Text]

    @property
    def field(
        self) -> RepeatedCompositeFieldContainer[FieldDescriptorProto]: ...

    @property
    def extension(
        self) -> RepeatedCompositeFieldContainer[FieldDescriptorProto]: ...

    @property
    def nested_type(
        self) -> RepeatedCompositeFieldContainer[DescriptorProto]: ...

    @property
    def enum_type(
        self) -> RepeatedCompositeFieldContainer[EnumDescriptorProto]: ...

    @property
    def extension_range(
        self) -> RepeatedCompositeFieldContainer[DescriptorProto.ExtensionRange]: ...

    @property
    def oneof_decl(
        self) -> RepeatedCompositeFieldContainer[OneofDescriptorProto]: ...

    @property
    def options(self) -> MessageOptions: ...

    @property
    def reserved_range(
        self) -> RepeatedCompositeFieldContainer[DescriptorProto.ReservedRange]: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 field: Optional[Iterable[FieldDescriptorProto]] = ...,
                 extension: Optional[Iterable[FieldDescriptorProto]] = ...,
                 nested_type: Optional[Iterable[DescriptorProto]] = ...,
                 enum_type: Optional[Iterable[EnumDescriptorProto]] = ...,
                 extension_range: Optional[Iterable[DescriptorProto.ExtensionRange]] = ...,
                 oneof_decl: Optional[Iterable[OneofDescriptorProto]] = ...,
                 options: Optional[MessageOptions] = ...,
                 reserved_range: Optional[Iterable[DescriptorProto.ReservedRange]] = ...,
                 reserved_name: Optional[Iterable[Text]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> DescriptorProto: ...


class ExtensionRangeOptions(Message):

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> ExtensionRangeOptions: ...


class FieldDescriptorProto(Message):

    class Type(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> FieldDescriptorProto.Type: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[FieldDescriptorProto.Type]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, FieldDescriptorProto.Type]]: ...
    TYPE_DOUBLE: Type
    TYPE_FLOAT: Type
    TYPE_INT64: Type
    TYPE_UINT64: Type
    TYPE_INT32: Type
    TYPE_FIXED64: Type
    TYPE_FIXED32: Type
    TYPE_BOOL: Type
    TYPE_STRING: Type
    TYPE_GROUP: Type
    TYPE_MESSAGE: Type
    TYPE_BYTES: Type
    TYPE_UINT32: Type
    TYPE_ENUM: Type
    TYPE_SFIXED32: Type
    TYPE_SFIXED64: Type
    TYPE_SINT32: Type
    TYPE_SINT64: Type

    class Label(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> FieldDescriptorProto.Label: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[FieldDescriptorProto.Label]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, FieldDescriptorProto.Label]]: ...
    LABEL_OPTIONAL: Label
    LABEL_REQUIRED: Label
    LABEL_REPEATED: Label
    name = ...  # type: Text
    number = ...  # type: int
    label = ...  # type: FieldDescriptorProto.Label
    type = ...  # type: FieldDescriptorProto.Type
    type_name = ...  # type: Text
    extendee = ...  # type: Text
    default_value = ...  # type: Text
    oneof_index = ...  # type: int
    json_name = ...  # type: Text

    @property
    def options(self) -> FieldOptions: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 number: Optional[int] = ...,
                 label: Optional[FieldDescriptorProto.Label] = ...,
                 type: Optional[FieldDescriptorProto.Type] = ...,
                 type_name: Optional[Text] = ...,
                 extendee: Optional[Text] = ...,
                 default_value: Optional[Text] = ...,
                 oneof_index: Optional[int] = ...,
                 json_name: Optional[Text] = ...,
                 options: Optional[FieldOptions] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FieldDescriptorProto: ...


class OneofDescriptorProto(Message):
    name = ...  # type: Text

    @property
    def options(self) -> OneofOptions: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 options: Optional[OneofOptions] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> OneofDescriptorProto: ...


class EnumDescriptorProto(Message):

    class EnumReservedRange(Message):
        start = ...  # type: int
        end = ...  # type: int

        def __init__(self,
                     start: Optional[int] = ...,
                     end: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(
            cls, s: bytes) -> EnumDescriptorProto.EnumReservedRange: ...
    name = ...  # type: Text
    reserved_name = ...  # type: RepeatedScalarFieldContainer[Text]

    @property
    def value(
        self) -> RepeatedCompositeFieldContainer[EnumValueDescriptorProto]: ...

    @property
    def options(self) -> EnumOptions: ...

    @property
    def reserved_range(
        self) -> RepeatedCompositeFieldContainer[EnumDescriptorProto.EnumReservedRange]: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 value: Optional[Iterable[EnumValueDescriptorProto]] = ...,
                 options: Optional[EnumOptions] = ...,
                 reserved_range: Optional[Iterable[EnumDescriptorProto.EnumReservedRange]] = ...,
                 reserved_name: Optional[Iterable[Text]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> EnumDescriptorProto: ...


class EnumValueDescriptorProto(Message):
    name = ...  # type: Text
    number = ...  # type: int

    @property
    def options(self) -> EnumValueOptions: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 number: Optional[int] = ...,
                 options: Optional[EnumValueOptions] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> EnumValueDescriptorProto: ...


class ServiceDescriptorProto(Message):
    name = ...  # type: Text

    @property
    def method(
        self) -> RepeatedCompositeFieldContainer[MethodDescriptorProto]: ...

    @property
    def options(self) -> ServiceOptions: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 method: Optional[Iterable[MethodDescriptorProto]] = ...,
                 options: Optional[ServiceOptions] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> ServiceDescriptorProto: ...


class MethodDescriptorProto(Message):
    name = ...  # type: Text
    input_type = ...  # type: Text
    output_type = ...  # type: Text
    client_streaming = ...  # type: bool
    server_streaming = ...  # type: bool

    @property
    def options(self) -> MethodOptions: ...

    def __init__(self,
                 name: Optional[Text] = ...,
                 input_type: Optional[Text] = ...,
                 output_type: Optional[Text] = ...,
                 options: Optional[MethodOptions] = ...,
                 client_streaming: Optional[bool] = ...,
                 server_streaming: Optional[bool] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> MethodDescriptorProto: ...


class FileOptions(Message):

    class OptimizeMode(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> FileOptions.OptimizeMode: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[FileOptions.OptimizeMode]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, FileOptions.OptimizeMode]]: ...
    SPEED: OptimizeMode
    CODE_SIZE: OptimizeMode
    LITE_RUNTIME: OptimizeMode
    java_package = ...  # type: Text
    java_outer_classname = ...  # type: Text
    java_multiple_files = ...  # type: bool
    java_generate_equals_and_hash = ...  # type: bool
    java_string_check_utf8 = ...  # type: bool
    optimize_for = ...  # type: FileOptions.OptimizeMode
    go_package = ...  # type: Text
    cc_generic_services = ...  # type: bool
    java_generic_services = ...  # type: bool
    py_generic_services = ...  # type: bool
    php_generic_services = ...  # type: bool
    deprecated = ...  # type: bool
    cc_enable_arenas = ...  # type: bool
    objc_class_prefix = ...  # type: Text
    csharp_namespace = ...  # type: Text
    swift_prefix = ...  # type: Text
    php_class_prefix = ...  # type: Text
    php_namespace = ...  # type: Text

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 java_package: Optional[Text] = ...,
                 java_outer_classname: Optional[Text] = ...,
                 java_multiple_files: Optional[bool] = ...,
                 java_generate_equals_and_hash: Optional[bool] = ...,
                 java_string_check_utf8: Optional[bool] = ...,
                 optimize_for: Optional[FileOptions.OptimizeMode] = ...,
                 go_package: Optional[Text] = ...,
                 cc_generic_services: Optional[bool] = ...,
                 java_generic_services: Optional[bool] = ...,
                 py_generic_services: Optional[bool] = ...,
                 php_generic_services: Optional[bool] = ...,
                 deprecated: Optional[bool] = ...,
                 cc_enable_arenas: Optional[bool] = ...,
                 objc_class_prefix: Optional[Text] = ...,
                 csharp_namespace: Optional[Text] = ...,
                 swift_prefix: Optional[Text] = ...,
                 php_class_prefix: Optional[Text] = ...,
                 php_namespace: Optional[Text] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FileOptions: ...


class MessageOptions(Message):
    message_set_wire_format = ...  # type: bool
    no_standard_descriptor_accessor = ...  # type: bool
    deprecated = ...  # type: bool
    map_entry = ...  # type: bool

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 message_set_wire_format: Optional[bool] = ...,
                 no_standard_descriptor_accessor: Optional[bool] = ...,
                 deprecated: Optional[bool] = ...,
                 map_entry: Optional[bool] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> MessageOptions: ...


class FieldOptions(Message):

    class CType(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> FieldOptions.CType: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[FieldOptions.CType]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, FieldOptions.CType]]: ...
    STRING: CType
    CORD: CType
    STRING_PIECE: CType

    class JSType(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> FieldOptions.JSType: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[FieldOptions.JSType]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, FieldOptions.JSType]]: ...
    JS_NORMAL: JSType
    JS_STRING: JSType
    JS_NUMBER: JSType
    ctype = ...  # type: FieldOptions.CType
    packed = ...  # type: bool
    jstype = ...  # type: FieldOptions.JSType
    lazy = ...  # type: bool
    deprecated = ...  # type: bool
    weak = ...  # type: bool

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 ctype: Optional[FieldOptions.CType] = ...,
                 packed: Optional[bool] = ...,
                 jstype: Optional[FieldOptions.JSType] = ...,
                 lazy: Optional[bool] = ...,
                 deprecated: Optional[bool] = ...,
                 weak: Optional[bool] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> FieldOptions: ...


class OneofOptions(Message):

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> OneofOptions: ...


class EnumOptions(Message):
    allow_alias = ...  # type: bool
    deprecated = ...  # type: bool

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 allow_alias: Optional[bool] = ...,
                 deprecated: Optional[bool] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> EnumOptions: ...


class EnumValueOptions(Message):
    deprecated = ...  # type: bool

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 deprecated: Optional[bool] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> EnumValueOptions: ...


class ServiceOptions(Message):
    deprecated = ...  # type: bool

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 deprecated: Optional[bool] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> ServiceOptions: ...


class MethodOptions(Message):

    class IdempotencyLevel(int):

        @classmethod
        def Name(cls, number: int) -> bytes: ...

        @classmethod
        def Value(cls, name: bytes) -> MethodOptions.IdempotencyLevel: ...

        @classmethod
        def keys(cls) -> List[bytes]: ...

        @classmethod
        def values(cls) -> List[MethodOptions.IdempotencyLevel]: ...

        @classmethod
        def items(cls) -> List[Tuple[bytes, MethodOptions.IdempotencyLevel]]: ...
    IDEMPOTENCY_UNKNOWN: IdempotencyLevel
    NO_SIDE_EFFECTS: IdempotencyLevel
    IDEMPOTENT: IdempotencyLevel
    deprecated = ...  # type: bool
    idempotency_level = ...  # type: MethodOptions.IdempotencyLevel

    @property
    def uninterpreted_option(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption]: ...

    def __init__(self,
                 deprecated: Optional[bool] = ...,
                 idempotency_level: Optional[MethodOptions.IdempotencyLevel] = ...,
                 uninterpreted_option: Optional[Iterable[UninterpretedOption]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> MethodOptions: ...


class UninterpretedOption(Message):

    class NamePart(Message):
        name_part = ...  # type: Text
        is_extension = ...  # type: bool

        def __init__(self,
                     name_part: Text,
                     is_extension: bool,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: bytes) -> UninterpretedOption.NamePart: ...
    identifier_value = ...  # type: Text
    positive_int_value = ...  # type: int
    negative_int_value = ...  # type: int
    double_value = ...  # type: float
    string_value = ...  # type: bytes
    aggregate_value = ...  # type: Text

    @property
    def name(
        self) -> RepeatedCompositeFieldContainer[UninterpretedOption.NamePart]: ...

    def __init__(self,
                 name: Optional[Iterable[UninterpretedOption.NamePart]] = ...,
                 identifier_value: Optional[Text] = ...,
                 positive_int_value: Optional[int] = ...,
                 negative_int_value: Optional[int] = ...,
                 double_value: Optional[float] = ...,
                 string_value: Optional[bytes] = ...,
                 aggregate_value: Optional[Text] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> UninterpretedOption: ...


class SourceCodeInfo(Message):

    class Location(Message):
        path = ...  # type: RepeatedScalarFieldContainer[int]
        span = ...  # type: RepeatedScalarFieldContainer[int]
        leading_comments = ...  # type: Text
        trailing_comments = ...  # type: Text
        leading_detached_comments = ...  # type: RepeatedScalarFieldContainer[Text]

        def __init__(self,
                     path: Optional[Iterable[int]] = ...,
                     span: Optional[Iterable[int]] = ...,
                     leading_comments: Optional[Text] = ...,
                     trailing_comments: Optional[Text] = ...,
                     leading_detached_comments: Optional[Iterable[Text]] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: bytes) -> SourceCodeInfo.Location: ...

    @property
    def location(
        self) -> RepeatedCompositeFieldContainer[SourceCodeInfo.Location]: ...

    def __init__(self,
                 location: Optional[Iterable[SourceCodeInfo.Location]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> SourceCodeInfo: ...


class GeneratedCodeInfo(Message):

    class Annotation(Message):
        path = ...  # type: RepeatedScalarFieldContainer[int]
        source_file = ...  # type: Text
        begin = ...  # type: int
        end = ...  # type: int

        def __init__(self,
                     path: Optional[Iterable[int]] = ...,
                     source_file: Optional[Text] = ...,
                     begin: Optional[int] = ...,
                     end: Optional[int] = ...,
                     ) -> None: ...

        @classmethod
        def FromString(cls, s: bytes) -> GeneratedCodeInfo.Annotation: ...

    @property
    def annotation(
        self) -> RepeatedCompositeFieldContainer[GeneratedCodeInfo.Annotation]: ...

    def __init__(self,
                 annotation: Optional[Iterable[GeneratedCodeInfo.Annotation]] = ...,
                 ) -> None: ...

    @classmethod
    def FromString(cls, s: bytes) -> GeneratedCodeInfo: ...



from typing import Any

number_re = ...  # type: Any
regex_type = ...  # type: Any
test_callable = ...  # type: Any

def test_odd(value): ...
def test_even(value): ...
def test_divisibleby(value, num): ...
def test_defined(value): ...
def test_undefined(value): ...
def test_none(value): ...
def test_lower(value): ...
def test_upper(value): ...
def test_string(value): ...
def test_mapping(value): ...
def test_number(value): ...
def test_sequence(value): ...
def test_equalto(value, other): ...
def test_sameas(value, other): ...
def test_iterable(value): ...
def test_escaped(value): ...

TESTS = ...  # type: Any



from typing import Any

SYM_EMPTY = ...  # type: Any

def list_or_args(keys, args): ...
def timestamp_to_datetime(response): ...
def string_keys_to_dict(key_string, callback): ...
def dict_merge(*dicts): ...
def parse_debug_object(response): ...
def parse_object(response, infotype): ...
def parse_info(response): ...

SENTINEL_STATE_TYPES = ...  # type: Any

def parse_sentinel_state(item): ...
def parse_sentinel_master(response): ...
def parse_sentinel_masters(response): ...
def parse_sentinel_slaves_and_sentinels(response): ...
def parse_sentinel_get_master(response): ...
def pairs_to_dict(response): ...
def pairs_to_dict_typed(response, type_info): ...
def zset_score_pairs(response, **options): ...
def sort_return_tuples(response, **options): ...
def int_or_none(response): ...
def float_or_none(response): ...
def bool_ok(response): ...
def parse_client_list(response, **options): ...
def parse_config_get(response, **options): ...
def parse_scan(response, **options): ...
def parse_hscan(response, **options): ...
def parse_zscan(response, **options): ...
def parse_slowlog_get(response, **options): ...

class StrictRedis:
    RESPONSE_CALLBACKS = ...  # type: Any
    @classmethod
    def from_url(cls, url, db=..., **kwargs): ...
    connection_pool = ...  # type: Any
    response_callbacks = ...  # type: Any
    def __init__(self, host=..., port=..., db=..., password=..., socket_timeout=..., socket_connect_timeout=...,
                 socket_keepalive=..., socket_keepalive_options=..., connection_pool=..., unix_socket_path=..., encoding=...,
                 encoding_errors=..., charset=..., errors=..., decode_responses=..., retry_on_timeout=..., ssl=...,
                 ssl_keyfile=..., ssl_certfile=..., ssl_cert_reqs=..., ssl_ca_certs=...) -> None: ...
    def set_response_callback(self, command, callback): ...
    def pipeline(self, transaction=..., shard_hint=...): ...
    def transaction(self, func, *watches, **kwargs): ...
    def lock(self, name, timeout=..., sleep=..., blocking_timeout=..., lock_class=..., thread_local=...): ...
    def pubsub(self, **kwargs): ...
    def execute_command(self, *args, **options): ...
    def parse_response(self, connection, command_name, **options): ...
    def bgrewriteaof(self): ...
    def bgsave(self): ...
    def client_kill(self, address): ...
    def client_list(self): ...
    def client_getname(self): ...
    def client_setname(self, name): ...
    def config_get(self, pattern=...): ...
    def config_set(self, name, value): ...
    def config_resetstat(self): ...
    def config_rewrite(self): ...
    def dbsize(self): ...
    def debug_object(self, key): ...
    def echo(self, value): ...
    def flushall(self): ...
    def flushdb(self): ...
    def info(self, section=...): ...
    def lastsave(self): ...
    def object(self, infotype, key): ...
    def ping(self): ...
    def save(self): ...
    def sentinel(self, *args): ...
    def sentinel_get_master_addr_by_name(self, service_name): ...
    def sentinel_master(self, service_name): ...
    def sentinel_masters(self): ...
    def sentinel_monitor(self, name, ip, port, quorum): ...
    def sentinel_remove(self, name): ...
    def sentinel_sentinels(self, service_name): ...
    def sentinel_set(self, name, option, value): ...
    def sentinel_slaves(self, service_name): ...
    def shutdown(self): ...
    def slaveof(self, host=..., port=...): ...
    def slowlog_get(self, num=...): ...
    def slowlog_len(self): ...
    def slowlog_reset(self): ...
    def time(self): ...
    def append(self, key, value): ...
    def bitcount(self, key, start=..., end=...): ...
    def bitop(self, operation, dest, *keys): ...
    def bitpos(self, key, bit, start=..., end=...): ...
    def decr(self, name, amount=...): ...
    def delete(self, *names): ...
    def __delitem__(self, name): ...
    def dump(self, name): ...
    def exists(self, name): ...
    __contains__ = ...  # type: Any
    def expire(self, name, time): ...
    def expireat(self, name, when): ...
    def get(self, name): ...
    def __getitem__(self, name): ...
    def getbit(self, name, offset): ...
    def getrange(self, key, start, end): ...
    def getset(self, name, value): ...
    def incr(self, name, amount=...): ...
    def incrby(self, name, amount=...): ...
    def incrbyfloat(self, name, amount=...): ...
    def keys(self, pattern=...): ...
    def mget(self, keys, *args): ...
    def mset(self, *args, **kwargs): ...
    def msetnx(self, *args, **kwargs): ...
    def move(self, name, db): ...
    def persist(self, name): ...
    def pexpire(self, name, time): ...
    def pexpireat(self, name, when): ...
    def psetex(self, name, time_ms, value): ...
    def pttl(self, name): ...
    def randomkey(self): ...
    def rename(self, src, dst): ...
    def renamenx(self, src, dst): ...
    def restore(self, name, ttl, value): ...
    def set(self, name, value, ex=..., px=..., nx=..., xx=...): ...
    def __setitem__(self, name, value): ...
    def setbit(self, name, offset, value): ...
    def setex(self, name, time, value): ...
    def setnx(self, name, value): ...
    def setrange(self, name, offset, value): ...
    def strlen(self, name): ...
    def substr(self, name, start, end=...): ...
    def ttl(self, name): ...
    def type(self, name): ...
    def watch(self, *names): ...
    def unwatch(self): ...
    def blpop(self, keys, timeout=...): ...
    def brpop(self, keys, timeout=...): ...
    def brpoplpush(self, src, dst, timeout=...): ...
    def lindex(self, name, index): ...
    def linsert(self, name, where, refvalue, value): ...
    def llen(self, name): ...
    def lpop(self, name): ...
    def lpush(self, name, *values): ...
    def lpushx(self, name, value): ...
    def lrange(self, name, start, end): ...
    def lrem(self, name, count, value): ...
    def lset(self, name, index, value): ...
    def ltrim(self, name, start, end): ...
    def rpop(self, name): ...
    def rpoplpush(self, src, dst): ...
    def rpush(self, name, *values): ...
    def rpushx(self, name, value): ...
    def sort(self, name, start=..., num=..., by=..., get=..., desc=..., alpha=..., store=..., groups=...): ...
    def scan(self, cursor=..., match=..., count=...): ...
    def scan_iter(self, match=..., count=...): ...
    def sscan(self, name, cursor=..., match=..., count=...): ...
    def sscan_iter(self, name, match=..., count=...): ...
    def hscan(self, name, cursor=..., match=..., count=...): ...
    def hscan_iter(self, name, match=..., count=...): ...
    def zscan(self, name, cursor=..., match=..., count=..., score_cast_func=...): ...
    def zscan_iter(self, name, match=..., count=..., score_cast_func=...): ...
    def sadd(self, name, *values): ...
    def scard(self, name): ...
    def sdiff(self, keys, *args): ...
    def sdiffstore(self, dest, keys, *args): ...
    def sinter(self, keys, *args): ...
    def sinterstore(self, dest, keys, *args): ...
    def sismember(self, name, value): ...
    def smembers(self, name): ...
    def smove(self, src, dst, value): ...
    def spop(self, name): ...
    def srandmember(self, name, number=...): ...
    def srem(self, name, *values): ...
    def sunion(self, keys, *args): ...
    def sunionstore(self, dest, keys, *args): ...
    def zadd(self, name, *args, **kwargs): ...
    def zcard(self, name): ...
    def zcount(self, name, min, max): ...
    def zincrby(self, name, value, amount=...): ...
    def zinterstore(self, dest, keys, aggregate=...): ...
    def zlexcount(self, name, min, max): ...
    def zrange(self, name, start, end, desc=..., withscores=..., score_cast_func=...): ...
    def zrangebylex(self, name, min, max, start=..., num=...): ...
    def zrangebyscore(self, name, min, max, start=..., num=..., withscores=..., score_cast_func=...): ...
    def zrank(self, name, value): ...
    def zrem(self, name, *values): ...
    def zremrangebylex(self, name, min, max): ...
    def zremrangebyrank(self, name, min, max): ...
    def zremrangebyscore(self, name, min, max): ...
    def zrevrange(self, name, start, end, withscores=..., score_cast_func=...): ...
    def zrevrangebyscore(self, name, max, min, start=..., num=..., withscores=..., score_cast_func=...): ...
    def zrevrank(self, name, value): ...
    def zscore(self, name, value): ...
    def zunionstore(self, dest, keys, aggregate=...): ...
    def pfadd(self, name, *values): ...
    def pfcount(self, name): ...
    def pfmerge(self, dest, *sources): ...
    def hdel(self, name, *keys): ...
    def hexists(self, name, key): ...
    def hget(self, name, key): ...
    def hgetall(self, name): ...
    def hincrby(self, name, key, amount=...): ...
    def hincrbyfloat(self, name, key, amount=...): ...
    def hkeys(self, name): ...
    def hlen(self, name): ...
    def hset(self, name, key, value): ...
    def hsetnx(self, name, key, value): ...
    def hmset(self, name, mapping): ...
    def hmget(self, name, keys, *args): ...
    def hvals(self, name): ...
    def publish(self, channel, message): ...
    def eval(self, script, numkeys, *keys_and_args): ...
    def evalsha(self, sha, numkeys, *keys_and_args): ...
    def script_exists(self, *args): ...
    def script_flush(self): ...
    def script_kill(self): ...
    def script_load(self, script): ...
    def register_script(self, script): ...

class Redis(StrictRedis):
    RESPONSE_CALLBACKS = ...  # type: Any
    def pipeline(self, transaction=..., shard_hint=...): ...
    def setex(self, name, value, time): ...
    def lrem(self, name, value, num=...): ...
    def zadd(self, name, *args, **kwargs): ...

class PubSub:
    PUBLISH_MESSAGE_TYPES = ...  # type: Any
    UNSUBSCRIBE_MESSAGE_TYPES = ...  # type: Any
    connection_pool = ...  # type: Any
    shard_hint = ...  # type: Any
    ignore_subscribe_messages = ...  # type: Any
    connection = ...  # type: Any
    encoding = ...  # type: Any
    encoding_errors = ...  # type: Any
    decode_responses = ...  # type: Any
    def __init__(self, connection_pool, shard_hint=..., ignore_subscribe_messages=...) -> None: ...
    def __del__(self): ...
    channels = ...  # type: Any
    patterns = ...  # type: Any
    def reset(self): ...
    def close(self): ...
    def on_connect(self, connection): ...
    def encode(self, value): ...
    @property
    def subscribed(self): ...
    def execute_command(self, *args, **kwargs): ...
    def parse_response(self, block=...): ...
    def psubscribe(self, *args, **kwargs): ...
    def punsubscribe(self, *args): ...
    def subscribe(self, *args, **kwargs): ...
    def unsubscribe(self, *args): ...
    def listen(self): ...
    def get_message(self, ignore_subscribe_messages=...): ...
    def handle_message(self, response, ignore_subscribe_messages=...): ...
    def run_in_thread(self, sleep_time=...): ...

class BasePipeline:
    UNWATCH_COMMANDS = ...  # type: Any
    connection_pool = ...  # type: Any
    connection = ...  # type: Any
    response_callbacks = ...  # type: Any
    transaction = ...  # type: Any
    shard_hint = ...  # type: Any
    watching = ...  # type: Any
    def __init__(self, connection_pool, response_callbacks, transaction, shard_hint) -> None: ...
    def __enter__(self): ...
    def __exit__(self, exc_type, exc_value, traceback): ...
    def __del__(self): ...
    def __len__(self): ...
    command_stack = ...  # type: Any
    scripts = ...  # type: Any
    explicit_transaction = ...  # type: Any
    def reset(self): ...
    def multi(self): ...
    def execute_command(self, *args, **kwargs): ...
    def immediate_execute_command(self, *args, **options): ...
    def pipeline_execute_command(self, *args, **options): ...
    def raise_first_error(self, commands, response): ...
    def annotate_exception(self, exception, number, command): ...
    def parse_response(self, connection, command_name, **options): ...
    def load_scripts(self): ...
    def execute(self, raise_on_error=...): ...
    def watch(self, *names): ...
    def unwatch(self): ...
    def script_load_for_pipeline(self, script): ...

class StrictPipeline(BasePipeline, StrictRedis): ...
class Pipeline(BasePipeline, Redis): ...

class Script:
    registered_client = ...  # type: Any
    script = ...  # type: Any
    sha = ...  # type: Any
    def __init__(self, registered_client, script) -> None: ...
    def __call__(self, keys=..., args=..., client=...): ...



# Stubs for tracemalloc (Python 3.4+)

import sys
from typing import List, Optional, Sequence, Tuple, Union, overload

def clear_traces() -> None: ...
def get_object_traceback(obj: object) -> Optional[Traceback]: ...
def get_traceback_limit() -> int: ...
def get_traced_memory() -> Tuple[int, int]: ...
def get_tracemalloc_memory() -> int: ...
def is_tracing() -> bool: ...
def start(nframe: int = ...) -> None: ...
def stop() -> None: ...
def take_snapshot() -> Snapshot: ...

if sys.version_info >= (3, 6):
    class DomainFilter:
        inclusive = ...  # type: bool
        domain = ...  # type: int
        def __init__(self, inclusive: bool, domain: int) -> None: ...

class Filter:
    if sys.version_info >= (3, 6):
        domain = ...  # type: Optional[int]
    inclusive = ...  # type: bool
    lineno = ...  # type: Optional[int]
    filename_pattern = ...  # type: str
    all_frames = ...  # type: bool
    def __init__(self, inclusive: bool, filename_pattern: str, lineno: Optional[int] = ..., all_frames: bool = ..., domain: Optional[int] = ...) -> None: ...

class Frame:
    filename = ...  # type: str
    lineno = ...  # type: int

class Snapshot:
    def compare_to(self, old_snapshot: Snapshot, key_type: str, cumulative: bool = ...) -> List[StatisticDiff]: ...
    def dump(self, filename: str) -> None: ...
    if sys.version_info >= (3, 6):
        def filter_traces(self, filters: Sequence[Union[DomainFilter, Filter]]) -> Snapshot: ...
    else:
        def filter_traces(self, filters: Sequence[Filter]) -> Snapshot: ...
    @classmethod
    def load(cls, filename: str) -> Snapshot: ...
    def statistics(self, key_type: str, cumulative: bool = ...) -> List[Statistic]: ...
    traceback_limit = ...  # type: int
    traces = ...  # type: Sequence[Trace]

class Statistic:
    count = ...  # type: int
    size = ...  # type: int
    traceback = ...  # type: Traceback

class StatisticDiff:
    count = ...  # type: int
    count_diff = ...  # type: int
    size = ...  # type: int
    size_diff = ...  # type: int
    traceback = ...  # type: Traceback

class Trace:
    size = ...  # type: int
    traceback = ...  # type: Traceback

class Traceback(Sequence[Frame]):
    def format(self, limit: Optional[int] = ...) -> List[str]: ...
    @overload
    def __getitem__(self, i: int) -> Frame: ...
    @overload
    def __getitem__(self, s: slice) -> Sequence[Frame]: ...
    def __len__(self) -> int: ...



# Stubs for email.mime.base (Python 3.4)

from typing import Optional, Tuple, Union
import email.message

_ParamsType = Union[str, None, Tuple[str, Optional[str], str]]

class MIMEBase(email.message.Message):
    def __init__(self, _maintype: str, _subtype: str,
                 **_params: _ParamsType) -> None: ...



"""Low-level infrastructure to find modules.

This build on fscache.py; find_sources.py builds on top of this.
"""

import ast
import collections
import functools
import os
import subprocess
import sys

from typing import Dict, List, NamedTuple, Optional, Set, Tuple

MYPY = False
if MYPY:
    from typing_extensions import Final

from mypy.defaults import PYTHON3_VERSION_MIN
from mypy.fscache import FileSystemCache
from mypy.options import Options
from mypy import sitepkgs

# Paths to be searched in find_module().
SearchPaths = NamedTuple(
    'SearchPaths',
    [('python_path', Tuple[str, ...]),  # where user code is found
     ('mypy_path', Tuple[str, ...]),  # from $MYPYPATH or config variable
     ('package_path', Tuple[str, ...]),  # from get_site_packages_dirs()
     ('typeshed_path', Tuple[str, ...]),  # paths in typeshed
     ])

# Package dirs are a two-tuple of path to search and whether to verify the module
OnePackageDir = Tuple[str, bool]
PackageDirs = List[OnePackageDir]

PYTHON_EXTENSIONS = ['.pyi', '.py']  # type: Final


class BuildSource:
    """A single source file."""

    def __init__(self, path: Optional[str], module: Optional[str],
                 text: Optional[str], base_dir: Optional[str] = None) -> None:
        self.path = path  # File where it's found (e.g. 'xxx/yyy/foo/bar.py')
        self.module = module or '__main__'  # Module name (e.g. 'foo.bar')
        self.text = text  # Source code, if initially supplied, else None
        self.base_dir = base_dir  # Directory where the package is rooted (e.g. 'xxx/yyy')

    def __repr__(self) -> str:
        return '<BuildSource path=%r module=%r has_text=%s>' % (self.path,
                                                                self.module,
                                                                self.text is not None)


class FindModuleCache:
    """Module finder with integrated cache.

    Module locations and some intermediate results are cached internally
    and can be cleared with the clear() method.

    All file system accesses are performed through a FileSystemCache,
    which is not ever cleared by this class. If necessary it must be
    cleared by client code.
    """

    def __init__(self,
                 search_paths: SearchPaths,
                 fscache: Optional[FileSystemCache] = None,
                 options: Optional[Options] = None) -> None:
        self.search_paths = search_paths
        self.fscache = fscache or FileSystemCache()
        # Cache find_lib_path_dirs: (dir_chain, search_paths) -> list(package_dirs, should_verify)
        self.dirs = {}  # type: Dict[Tuple[str, Tuple[str, ...]], PackageDirs]
        # Cache find_module: id -> result
        self.results = {}  # type: Dict[str, Optional[str]]
        self.ns_ancestors = {}  # type: Dict[str, str]
        self.options = options

    def clear(self) -> None:
        self.results.clear()
        self.dirs.clear()
        self.ns_ancestors.clear()

    def find_lib_path_dirs(self, dir_chain: str, lib_path: Tuple[str, ...]) -> PackageDirs:
        # Cache some repeated work within distinct find_module calls: finding which
        # elements of lib_path have even the subdirectory they'd need for the module
        # to exist. This is shared among different module ids when they differ only
        # in the last component.
        # This is run for the python_path, mypy_path, and typeshed_path search paths
        key = (dir_chain, lib_path)
        if key not in self.dirs:
            self.dirs[key] = self._find_lib_path_dirs(dir_chain, lib_path)
        return self.dirs[key]

    def _find_lib_path_dirs(self, dir_chain: str, lib_path: Tuple[str, ...]) -> PackageDirs:
        dirs = []
        for pathitem in lib_path:
            # e.g., '/usr/lib/python3.4/foo/bar'
            dir = os.path.normpath(os.path.join(pathitem, dir_chain))
            if self.fscache.isdir(dir):
                dirs.append((dir, True))
        return dirs

    def find_module(self, id: str) -> Optional[str]:
        """Return the path of the module source file, or None if not found."""
        if id not in self.results:
            self.results[id] = self._find_module(id)
        return self.results[id]

    def _find_module_non_stub_helper(self, components: List[str],
                                     pkg_dir: str) -> Optional[OnePackageDir]:
        dir_path = pkg_dir
        for index, component in enumerate(components):
            dir_path = os.path.join(dir_path, component)
            if self.fscache.isfile(os.path.join(dir_path, 'py.typed')):
                return os.path.join(pkg_dir, *components[:-1]), index == 0
        return None

    def _update_ns_ancestors(self, components: List[str], match: Tuple[str, bool]) -> None:
        path, verify = match
        for i in range(1, len(components)):
            pkg_id = '.'.join(components[:-i])
            if pkg_id not in self.ns_ancestors and self.fscache.isdir(path):
                self.ns_ancestors[pkg_id] = path
            path = os.path.dirname(path)

    def _find_module(self, id: str) -> Optional[str]:
        fscache = self.fscache

        # If we're looking for a module like 'foo.bar.baz', it's likely that most of the
        # many elements of lib_path don't even have a subdirectory 'foo/bar'.  Discover
        # that only once and cache it for when we look for modules like 'foo.bar.blah'
        # that will require the same subdirectory.
        components = id.split('.')
        dir_chain = os.sep.join(components[:-1])  # e.g., 'foo/bar'
        # TODO (ethanhs): refactor each path search to its own method with lru_cache

        # We have two sets of folders so that we collect *all* stubs folders and
        # put them in the front of the search path
        third_party_inline_dirs = []  # type: PackageDirs
        third_party_stubs_dirs = []  # type: PackageDirs
        # Third-party stub/typed packages
        for pkg_dir in self.search_paths.package_path:
            stub_name = components[0] + '-stubs'
            stub_dir = os.path.join(pkg_dir, stub_name)
            if fscache.isdir(stub_dir):
                stub_typed_file = os.path.join(stub_dir, 'py.typed')
                stub_components = [stub_name] + components[1:]
                path = os.path.join(pkg_dir, *stub_components[:-1])
                if fscache.isdir(path):
                    if fscache.isfile(stub_typed_file):
                        # Stub packages can have a py.typed file, which must include
                        # 'partial\n' to make the package partial
                        # Partial here means that mypy should look at the runtime
                        # package if installed.
                        if fscache.read(stub_typed_file).decode().strip() == 'partial':
                            runtime_path = os.path.join(pkg_dir, dir_chain)
                            third_party_inline_dirs.append((runtime_path, True))
                            # if the package is partial, we don't verify the module, as
                            # the partial stub package may not have a __init__.pyi
                            third_party_stubs_dirs.append((path, False))
                        else:
                            # handle the edge case where people put a py.typed file
                            # in a stub package, but it isn't partial
                            third_party_stubs_dirs.append((path, True))
                    else:
                        third_party_stubs_dirs.append((path, True))
            non_stub_match = self._find_module_non_stub_helper(components, pkg_dir)
            if non_stub_match:
                third_party_inline_dirs.append(non_stub_match)
                self._update_ns_ancestors(components, non_stub_match)
        if self.options and self.options.use_builtins_fixtures:
            # Everything should be in fixtures.
            third_party_inline_dirs.clear()
            third_party_stubs_dirs.clear()
        python_mypy_path = self.search_paths.mypy_path + self.search_paths.python_path
        candidate_base_dirs = self.find_lib_path_dirs(dir_chain, python_mypy_path) + \
            third_party_stubs_dirs + third_party_inline_dirs + \
            self.find_lib_path_dirs(dir_chain, self.search_paths.typeshed_path)

        # If we're looking for a module like 'foo.bar.baz', then candidate_base_dirs now
        # contains just the subdirectories 'foo/bar' that actually exist under the
        # elements of lib_path.  This is probably much shorter than lib_path itself.
        # Now just look for 'baz.pyi', 'baz/__init__.py', etc., inside those directories.
        seplast = os.sep + components[-1]  # so e.g. '/baz'
        sepinit = os.sep + '__init__'
        near_misses = []  # Collect near misses for namespace mode (see below).
        for base_dir, verify in candidate_base_dirs:
            base_path = base_dir + seplast  # so e.g. '/usr/lib/python3.4/foo/bar/baz'
            # Prefer package over module, i.e. baz/__init__.py* over baz.py*.
            for extension in PYTHON_EXTENSIONS:
                path = base_path + sepinit + extension
                path_stubs = base_path + '-stubs' + sepinit + extension
                if fscache.isfile_case(path):
                    if verify and not verify_module(fscache, id, path):
                        near_misses.append(path)
                        continue
                    return path
                elif fscache.isfile_case(path_stubs):
                    if verify and not verify_module(fscache, id, path_stubs):
                        near_misses.append(path_stubs)
                        continue
                    return path_stubs
                elif self.options and self.options.namespace_packages and fscache.isdir(base_path):
                    near_misses.append(base_path)
            # No package, look for module.
            for extension in PYTHON_EXTENSIONS:
                path = base_path + extension
                if fscache.isfile_case(path):
                    if verify and not verify_module(fscache, id, path):
                        near_misses.append(path)
                        continue
                    return path

        # In namespace mode, re-check those entries that had 'verify'.
        # Assume search path entries xxx, yyy and zzz, and we're
        # looking for foo.bar.baz.  Suppose near_misses has:
        #
        # - xxx/foo/bar/baz.py
        # - yyy/foo/bar/baz/__init__.py
        # - zzz/foo/bar/baz.pyi
        #
        # If any of the foo directories has __init__.py[i], it wins.
        # Else, we look for foo/bar/__init__.py[i], etc.  If there are
        # none, the first hit wins.  Note that this does not take into
        # account whether the lowest-level module is a file (baz.py),
        # a package (baz/__init__.py), or a stub file (baz.pyi) -- for
        # these the first one encountered along the search path wins.
        #
        # The helper function highest_init_level() returns an int that
        # indicates the highest level at which a __init__.py[i] file
        # is found; if no __init__ was found it returns 0, if we find
        # only foo/bar/__init__.py it returns 1, and if we have
        # foo/__init__.py it returns 2 (regardless of what's in
        # foo/bar).  It doesn't look higher than that.
        if self.options and self.options.namespace_packages and near_misses:
            levels = [highest_init_level(fscache, id, path) for path in near_misses]
            index = levels.index(max(levels))
            return near_misses[index]

        # Finally, we may be asked to produce an ancestor for an
        # installed package with a py.typed marker that is a
        # subpackage of a namespace package.  We only fess up to these
        # if we would otherwise return "not found".
        return self.ns_ancestors.get(id)

    def find_modules_recursive(self, module: str) -> List[BuildSource]:
        module_path = self.find_module(module)
        if not module_path:
            return []
        result = [BuildSource(module_path, module, None)]
        if module_path.endswith(('__init__.py', '__init__.pyi')):
            # Subtle: this code prefers the .pyi over the .py if both
            # exists, and also prefers packages over modules if both x/
            # and x.py* exist.  How?  We sort the directory items, so x
            # comes before x.py and x.pyi.  But the preference for .pyi
            # over .py is encoded in find_module(); even though we see
            # x.py before x.pyi, find_module() will find x.pyi first.  We
            # use hits to avoid adding it a second time when we see x.pyi.
            # This also avoids both x.py and x.pyi when x/ was seen first.
            hits = set()  # type: Set[str]
            for item in sorted(self.fscache.listdir(os.path.dirname(module_path))):
                abs_path = os.path.join(os.path.dirname(module_path), item)
                if os.path.isdir(abs_path) and \
                        (os.path.isfile(os.path.join(abs_path, '__init__.py')) or
                        os.path.isfile(os.path.join(abs_path, '__init__.pyi'))):
                    hits.add(item)
                    result += self.find_modules_recursive(module + '.' + item)
                elif item != '__init__.py' and item != '__init__.pyi' and \
                        item.endswith(('.py', '.pyi')):
                    mod = item.split('.')[0]
                    if mod not in hits:
                        hits.add(mod)
                        result += self.find_modules_recursive(module + '.' + mod)
        return result


def verify_module(fscache: FileSystemCache, id: str, path: str) -> bool:
    """Check that all packages containing id have a __init__ file."""
    if path.endswith(('__init__.py', '__init__.pyi')):
        path = os.path.dirname(path)
    for i in range(id.count('.')):
        path = os.path.dirname(path)
        if not any(fscache.isfile_case(os.path.join(path, '__init__{}'.format(extension)))
                   for extension in PYTHON_EXTENSIONS):
            return False
    return True


def highest_init_level(fscache: FileSystemCache, id: str, path: str) -> int:
    """Compute the highest level where an __init__ file is found."""
    if path.endswith(('__init__.py', '__init__.pyi')):
        path = os.path.dirname(path)
    level = 0
    for i in range(id.count('.')):
        path = os.path.dirname(path)
        if any(fscache.isfile_case(os.path.join(path, '__init__{}'.format(extension)))
               for extension in PYTHON_EXTENSIONS):
            level = i + 1
    return level


def mypy_path() -> List[str]:
    path_env = os.getenv('MYPYPATH')
    if not path_env:
        return []
    return path_env.split(os.pathsep)


def default_lib_path(data_dir: str,
                     pyversion: Tuple[int, int],
                     custom_typeshed_dir: Optional[str]) -> List[str]:
    """Return default standard library search paths."""
    # IDEA: Make this more portable.
    path = []  # type: List[str]

    if custom_typeshed_dir:
        typeshed_dir = custom_typeshed_dir
    else:
        auto = os.path.join(data_dir, 'stubs-auto')
        if os.path.isdir(auto):
            data_dir = auto
        typeshed_dir = os.path.join(data_dir, "typeshed")
    if pyversion[0] == 3:
        # We allow a module for e.g. version 3.5 to be in 3.4/. The assumption
        # is that a module added with 3.4 will still be present in Python 3.5.
        versions = ["%d.%d" % (pyversion[0], minor)
                    for minor in reversed(range(PYTHON3_VERSION_MIN[1], pyversion[1] + 1))]
    else:
        # For Python 2, we only have stubs for 2.7
        versions = ["2.7"]
    # E.g. for Python 3.6, try 3.6/, 3.5/, 3.4/, 3/, 2and3/.
    for v in versions + [str(pyversion[0]), '2and3']:
        for lib_type in ['stdlib', 'third_party']:
            stubdir = os.path.join(typeshed_dir, lib_type, v)
            if os.path.isdir(stubdir):
                path.append(stubdir)

    # Add fallback path that can be used if we have a broken installation.
    if sys.platform != 'win32':
        path.append('/usr/local/lib/mypy')
    if not path:
        print("Could not resolve typeshed subdirectories. If you are using mypy\n"
              "from source, you need to run \"git submodule update --init\".\n"
              "Otherwise your mypy install is broken.\nPython executable is located at "
              "{0}.\nMypy located at {1}".format(sys.executable, data_dir), file=sys.stderr)
        sys.exit(1)
    return path


@functools.lru_cache(maxsize=None)
def get_site_packages_dirs(python_executable: Optional[str]) -> Tuple[List[str], List[str]]:
    """Find package directories for given python.

    This runs a subprocess call, which generates a list of the egg directories, and the site
    package directories. To avoid repeatedly calling a subprocess (which can be slow!) we
    lru_cache the results."""
    def make_abspath(path: str, root: str) -> str:
        """Take a path and make it absolute relative to root if not already absolute."""
        if os.path.isabs(path):
            return os.path.normpath(path)
        else:
            return os.path.join(root, os.path.normpath(path))

    if python_executable is None:
        return [], []
    if python_executable == sys.executable:
        # Use running Python's package dirs
        site_packages = sitepkgs.getsitepackages()
    else:
        # Use subprocess to get the package directory of given Python
        # executable
        site_packages = ast.literal_eval(
            subprocess.check_output([python_executable, sitepkgs.__file__],
            stderr=subprocess.PIPE).decode())
    egg_dirs = []
    for dir in site_packages:
        pth = os.path.join(dir, 'easy-install.pth')
        if os.path.isfile(pth):
            with open(pth) as f:
                egg_dirs.extend([make_abspath(d.rstrip(), dir) for d in f.readlines()])
    return egg_dirs, site_packages


def compute_search_paths(sources: List[BuildSource],
                         options: Options,
                         data_dir: str,
                         alt_lib_path: Optional[str] = None) -> SearchPaths:
    """Compute the search paths as specified in PEP 561.

    There are the following 4 members created:
    - User code (from `sources`)
    - MYPYPATH (set either via config or environment variable)
    - installed package directories (which will later be split into stub-only and inline)
    - typeshed
     """
    # Determine the default module search path.
    lib_path = collections.deque(
        default_lib_path(data_dir,
                         options.python_version,
                         custom_typeshed_dir=options.custom_typeshed_dir))

    if options.use_builtins_fixtures:
        # Use stub builtins (to speed up test cases and to make them easier to
        # debug).  This is a test-only feature, so assume our files are laid out
        # as in the source tree.
        # We also need to allow overriding where to look for it. Argh.
        root_dir = os.getenv('MYPY_TEST_PREFIX', None)
        if not root_dir:
            root_dir = os.path.dirname(os.path.dirname(__file__))
        lib_path.appendleft(os.path.join(root_dir, 'test-data', 'unit', 'lib-stub'))
    # alt_lib_path is used by some tests to bypass the normal lib_path mechanics.
    # If we don't have one, grab directories of source files.
    python_path = []  # type: List[str]
    if not alt_lib_path:
        for source in sources:
            # Include directory of the program file in the module search path.
            if source.base_dir:
                dir = source.base_dir
                if dir not in python_path:
                    python_path.append(dir)

        # Do this even if running as a file, for sanity (mainly because with
        # multiple builds, there could be a mix of files/modules, so its easier
        # to just define the semantics that we always add the current director
        # to the lib_path
        # TODO: Don't do this in some cases; for motivation see see
        # https://github.com/python/mypy/issues/4195#issuecomment-341915031
        if options.bazel:
            dir = '.'
        else:
            dir = os.getcwd()
        if dir not in lib_path:
            python_path.insert(0, dir)

    # Start with a MYPYPATH environment variable at the front of the mypy_path, if defined.
    mypypath = mypy_path()

    # Add a config-defined mypy path.
    mypypath.extend(options.mypy_path)

    # If provided, insert the caller-supplied extra module path to the
    # beginning (highest priority) of the search path.
    if alt_lib_path:
        mypypath.insert(0, alt_lib_path)

    egg_dirs, site_packages = get_site_packages_dirs(options.python_executable)
    for site_dir in site_packages:
        assert site_dir not in lib_path
        if site_dir in mypypath:
            print("{} is in the MYPYPATH. Please remove it.".format(site_dir), file=sys.stderr)
            sys.exit(1)
        elif site_dir in python_path:
            print("{} is in the PYTHONPATH. Please change directory"
                  " so it is not.".format(site_dir),
                  file=sys.stderr)
            sys.exit(1)

    return SearchPaths(tuple(reversed(python_path)),
                       tuple(mypypath),
                       tuple(egg_dirs + site_packages),
                       tuple(lib_path))



# Based on http://docs.python.org/3.5/library/configparser.html and on
# reading configparser.py.

import sys
from typing import (AbstractSet, MutableMapping, Mapping, Dict, Sequence, List,
                    Union, Iterable, Iterator, Callable, Any, IO, overload,
                    Optional, Pattern, Type, TypeVar)
# Types only used in type comments only
from typing import Optional, Tuple  # noqa

if sys.version_info >= (3, 6):
    from os import PathLike

# Internal type aliases
_section = Mapping[str, str]
_parser = MutableMapping[str, _section]
_converter = Callable[[str], Any]
_converters = Dict[str, _converter]
_T = TypeVar('_T')

if sys.version_info >= (3, 7):
    _Path = Union[str, bytes, PathLike[str]]
elif sys.version_info >= (3, 6):
    _Path = Union[str, PathLike[str]]
else:
    _Path = str

DEFAULTSECT: str
MAX_INTERPOLATION_DEPTH: int

class Interpolation:
    def before_get(self, parser: _parser,
                   section: str,
                   option: str,
                   value: str,
                   defaults: _section) -> str: ...

    def before_set(self, parser: _parser,
                   section: str,
                   option: str,
                   value: str) -> str: ...

    def before_read(self, parser: _parser,
                    section: str,
                    option: str,
                    value: str) -> str: ...

    def before_write(self, parser: _parser,
                     section: str,
                     option: str,
                     value: str) -> str: ...


class BasicInterpolation(Interpolation): ...
class ExtendedInterpolation(Interpolation): ...
class LegacyInterpolation(Interpolation): ...


class RawConfigParser(_parser):
    def __init__(self,
                 defaults: Optional[_section] = ...,
                 dict_type: Type[Mapping[str, str]] = ...,
                 allow_no_value: bool = ...,
                 *,
                 delimiters: Sequence[str] = ...,
                 comment_prefixes: Sequence[str] = ...,
                 inline_comment_prefixes: Optional[Sequence[str]] = ...,
                 strict: bool = ...,
                 empty_lines_in_values: bool = ...,
                 default_section: str = ...,
                 interpolation: Optional[Interpolation] = ...) -> None: ...

    def __len__(self) -> int: ...

    def __getitem__(self, section: str) -> SectionProxy: ...

    def __setitem__(self, section: str, options: _section) -> None: ...

    def __delitem__(self, section: str) -> None: ...

    def __iter__(self) -> Iterator[str]: ...

    def defaults(self) -> _section: ...

    def sections(self) -> List[str]: ...

    def add_section(self, section: str) -> None: ...

    def has_section(self, section: str) -> bool: ...

    def options(self, section: str) -> List[str]: ...

    def has_option(self, section: str, option: str) -> bool: ...

    def read(self, filenames: Union[_Path, Iterable[_Path]],
             encoding: Optional[str] = ...) -> List[str]: ...
    def read_file(self, f: Iterable[str], source: Optional[str] = ...) -> None: ...
    def read_string(self, string: str, source: str = ...) -> None: ...
    def read_dict(self, dictionary: Mapping[str, Mapping[str, Any]],
                  source: str = ...) -> None: ...
    def readfp(self, fp: Iterable[str], filename: Optional[str] = ...) -> None: ...

    # These get* methods are partially applied (with the same names) in
    # SectionProxy; the stubs should be kept updated together
    def getint(self, section: str, option: str, *, raw: bool = ..., vars: _section = ..., fallback: int = ...) -> int: ...

    def getfloat(self, section: str, option: str, *, raw: bool = ..., vars: _section = ..., fallback: float = ...) -> float: ...

    def getboolean(self, section: str, option: str, *, raw: bool = ..., vars: _section = ..., fallback: bool = ...) -> bool: ...

    def _get_conv(self, section: str, option: str, conv: Callable[[str], _T], *, raw: bool = ..., vars: _section = ..., fallback: _T = ...) -> _T: ...

    # This is incompatible with MutableMapping so we ignore the type
    def get(self, section: str, option: str, *, raw: bool = ..., vars: _section = ..., fallback: str = ...) -> str:  # type: ignore
        ...

    @overload
    def items(self, *, raw: bool = ..., vars: _section = ...) -> AbstractSet[Tuple[str, SectionProxy]]: ...

    @overload
    def items(self, section: str, raw: bool = ..., vars: _section = ...) -> List[Tuple[str, str]]: ...

    def set(self, section: str, option: str, value: str) -> None: ...

    def write(self,
              fileobject: IO[str],
              space_around_delimiters: bool = ...) -> None: ...

    def remove_option(self, section: str, option: str) -> bool: ...

    def remove_section(self, section: str) -> bool: ...

    def optionxform(self, option: str) -> str: ...


class ConfigParser(RawConfigParser):
    def __init__(self,
                 defaults: Optional[_section] = ...,
                 dict_type: Type[Mapping[str, str]] = ...,
                 allow_no_value: bool = ...,
                 delimiters: Sequence[str] = ...,
                 comment_prefixes: Sequence[str] = ...,
                 inline_comment_prefixes: Optional[Sequence[str]] = ...,
                 strict: bool = ...,
                 empty_lines_in_values: bool = ...,
                 default_section: str = ...,
                 interpolation: Optional[Interpolation] = ...,
                 converters: _converters = ...) -> None: ...

class SafeConfigParser(ConfigParser): ...

class SectionProxy(MutableMapping[str, str]):
    def __init__(self, parser: RawConfigParser, name: str) -> None: ...
    def __getitem__(self, key: str) -> str: ...
    def __setitem__(self, key: str, value: str) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __contains__(self, key: object) -> bool: ...
    def __len__(self) -> int: ...
    def __iter__(self) -> Iterator[str]: ...
    @property
    def parser(self) -> RawConfigParser: ...
    @property
    def name(self) -> str: ...
    def get(self, option: str, fallback: Optional[str] = ..., *, raw: bool = ..., vars: Optional[_section] = ..., **kwargs: Any) -> str: ...  # type: ignore

    # These are partially-applied version of the methods with the same names in
    # RawConfigParser; the stubs should be kept updated together
    def getint(self, option: str, *, raw: bool = ..., vars: _section = ..., fallback: int = ...) -> int: ...
    def getfloat(self, option: str, *, raw: bool = ..., vars: _section = ..., fallback: float = ...) -> float: ...
    def getboolean(self, option: str, *, raw: bool = ..., vars: _section = ..., fallback: bool = ...) -> bool: ...

    # SectionProxy can have arbitrary attributes when custon converters are used
    def __getattr__(self, key: str) -> Callable[..., Any]: ...

class ConverterMapping(MutableMapping[str, Optional[_converter]]):
    GETTERCRE: Pattern
    def __init__(self, parser: RawConfigParser) -> None: ...
    def __getitem__(self, key: str) -> _converter: ...
    def __setitem__(self, key: str, value: Optional[_converter]) -> None: ...
    def __delitem__(self, key: str) -> None: ...
    def __iter__(self) -> Iterator[str]: ...
    def __len__(self) -> int: ...


class Error(Exception): ...


class NoSectionError(Error): ...


class DuplicateSectionError(Error):
    section = ...  # type: str
    source = ...   # type: Optional[str]
    lineno = ...   # type: Optional[int]


class DuplicateOptionError(Error):
    section = ...  # type: str
    option = ...   # type: str
    source = ...   # type: Optional[str]
    lineno = ...   # type: Optional[int]


class NoOptionError(Error):
    section = ...  # type: str
    option = ...   # type: str


class InterpolationError(Error):
    section = ...  # type: str
    option = ...   # type: str


class InterpolationDepthError(InterpolationError): ...


class InterpolationMissingOptionError(InterpolationError):
    reference = ...  # type: str


class InterpolationSyntaxError(InterpolationError): ...


class ParsingError(Error):
    source = ...  # type: str
    errors = ...  # type: Sequence[Tuple[int, str]]


class MissingSectionHeaderError(ParsingError):
    lineno = ...  # type: int
    line = ...    # type: str



"""Hack for handling non-mypyc compiled plugins with a mypyc-compiled mypy"""

from typing import Optional, Callable, Any, Dict
from mypy.options import Options
from mypy.types import Type, CallableType
from mypy.nodes import SymbolTableNode, MypyFile
from mypy.lookup import lookup_fully_qualified

MYPY = False
if MYPY:
    import mypy.plugin


class InterpretedPlugin:
    """Base class of type checker plugins as exposed to external code.

    This is a hack around mypyc not currently supporting interpreted subclasses
    of compiled classes.
    mypy.plugin will arrange for interpreted code to be find this class when it looks
    for Plugin, and this class has a __new__ method that returns a WrapperPlugin object
    that proxies to this interpreted version.
    """

    def __new__(cls, *args: Any, **kwargs: Any) -> 'mypy.plugin.Plugin':
        from mypy.plugin import WrapperPlugin
        plugin = object.__new__(cls)  # type: ignore
        plugin.__init__(*args, **kwargs)
        return WrapperPlugin(plugin)

    def __init__(self, options: Options) -> None:
        self.options = options
        self.python_version = options.python_version
        self._modules = None  # type: Optional[Dict[str, MypyFile]]

    def set_modules(self, modules: Dict[str, MypyFile]) -> None:
        self._modules = modules

    def lookup_fully_qualified(self, fullname: str) -> Optional[SymbolTableNode]:
        assert self._modules is not None
        return lookup_fully_qualified(fullname, self._modules)

    def get_type_analyze_hook(self, fullname: str
                              ) -> Optional[Callable[['mypy.plugin.AnalyzeTypeContext'], Type]]:
        return None

    def get_function_hook(self, fullname: str
                          ) -> Optional[Callable[['mypy.plugin.FunctionContext'], Type]]:
        return None

    def get_method_signature_hook(self, fullname: str
                                  ) -> Optional[Callable[['mypy.plugin.MethodSigContext'],
                                                         CallableType]]:
        return None

    def get_method_hook(self, fullname: str
                        ) -> Optional[Callable[['mypy.plugin.MethodContext'], Type]]:
        return None

    def get_attribute_hook(self, fullname: str
                           ) -> Optional[Callable[['mypy.plugin.AttributeContext'], Type]]:
        return None

    def get_class_decorator_hook(self, fullname: str
                                 ) -> Optional[Callable[['mypy.plugin.ClassDefContext'], None]]:
        return None

    def get_metaclass_hook(self, fullname: str
                           ) -> Optional[Callable[['mypy.plugin.ClassDefContext'], None]]:
        return None

    def get_base_class_hook(self, fullname: str
                            ) -> Optional[Callable[['mypy.plugin.ClassDefContext'], None]]:
        return None

    def get_customize_class_mro_hook(self, fullname: str
                                     ) -> Optional[Callable[['mypy.plugin.ClassDefContext'],
                                                            None]]:
        return None

    def get_dynamic_class_hook(self, fullname: str
                               ) -> Optional[Callable[['mypy.plugin.DynamicClassDefContext'],
                                                      None]]:
        return None



from typing import Any

poll = ...  # type: Any
select = ...  # type: Any
HAS_IPV6 = ...  # type: bool

def is_connection_dropped(conn): ...
def create_connection(address, timeout=..., source_address=..., socket_options=...): ...



from typing import Any, Optional

class HashAlgo:
    digest_size = ...  # type: Any
    block_size = ...  # type: Any
    def __init__(self, hashFactory, data: Optional[Any] = ...) -> None: ...
    def update(self, data): ...
    def digest(self): ...
    def hexdigest(self): ...
    def copy(self): ...
    def new(self, data: Optional[Any] = ...): ...



# Stubs for functools (Python 2.7)

# NOTE: These are incomplete!

from abc import ABCMeta, abstractmethod
from typing import Any, Callable, Generic, Dict, Iterable, Optional, Sequence, Tuple, TypeVar, overload
from collections import namedtuple

_AnyCallable = Callable[..., Any]

_T = TypeVar("_T")
_S = TypeVar("_S")
@overload
def reduce(function: Callable[[_T, _T], _T],
           sequence: Iterable[_T]) -> _T: ...
@overload
def reduce(function: Callable[[_T, _S], _T],
           sequence: Iterable[_S], initial: _T) -> _T: ...

WRAPPER_ASSIGNMENTS = ...  # type: Sequence[str]
WRAPPER_UPDATES = ...  # type: Sequence[str]

def update_wrapper(wrapper: _AnyCallable, wrapped: _AnyCallable, assigned: Sequence[str] = ...,
                   updated: Sequence[str] = ...) -> _AnyCallable: ...
def wraps(wrapped: _AnyCallable, assigned: Sequence[str] = ..., updated: Sequence[str] = ...) -> Callable[[_AnyCallable], _AnyCallable]: ...
def total_ordering(cls: type) -> type: ...
def cmp_to_key(mycmp: Callable[[_T, _T], int]) -> Callable[[_T], Any]: ...

class partial(Generic[_T]):
    func = ...  # Callable[..., _T]
    args = ...  # type: Tuple[Any, ...]
    keywords = ...  # type: Dict[str, Any]
    def __init__(self, func: Callable[..., _T], *args: Any, **kwargs: Any) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> _T: ...



"""Conversion of parse tree nodes to strings."""

import re
import os

from typing import Any, List, Tuple, Optional, Union, Sequence

from mypy.util import short_type, IdMapper
import mypy.nodes
from mypy.visitor import NodeVisitor


class StrConv(NodeVisitor[str]):
    """Visitor for converting a node to a human-readable string.

    For example, an MypyFile node from program '1' is converted into
    something like this:

      MypyFile:1(
        fnam
        ExpressionStmt:1(
          IntExpr(1)))
    """

    def __init__(self, show_ids: bool = False) -> None:
        self.show_ids = show_ids
        self.id_mapper = None  # type: Optional[IdMapper]
        if show_ids:
            self.id_mapper = IdMapper()

    def get_id(self, o: object) -> Optional[int]:
        if self.id_mapper:
            return self.id_mapper.id(o)
        return None

    def format_id(self, o: object) -> str:
        if self.id_mapper:
            return '<{}>'.format(self.get_id(o))
        else:
            return ''

    def dump(self, nodes: Sequence[object], obj: 'mypy.nodes.Context') -> str:
        """Convert a list of items to a multiline pretty-printed string.

        The tag is produced from the type name of obj and its line
        number. See mypy.util.dump_tagged for a description of the nodes
        argument.
        """
        tag = short_type(obj) + ':' + str(obj.get_line())
        if self.show_ids:
            assert self.id_mapper is not None
            tag += '<{}>'.format(self.get_id(obj))
        return dump_tagged(nodes, tag, self)

    def func_helper(self, o: 'mypy.nodes.FuncItem') -> List[object]:
        """Return a list in a format suitable for dump() that represents the
        arguments and the body of a function. The caller can then decorate the
        array with information specific to methods, global functions or
        anonymous functions.
        """
        args = []  # type: List[Union[mypy.nodes.Var, Tuple[str, List[mypy.nodes.Node]]]]
        extra = []  # type: List[Tuple[str, List[mypy.nodes.Var]]]
        for arg in o.arguments:
            kind = arg.kind  # type: int
            if kind in (mypy.nodes.ARG_POS, mypy.nodes.ARG_NAMED):
                args.append(arg.variable)
            elif kind in (mypy.nodes.ARG_OPT, mypy.nodes.ARG_NAMED_OPT):
                assert arg.initializer is not None
                args.append(('default', [arg.variable, arg.initializer]))
            elif kind == mypy.nodes.ARG_STAR:
                extra.append(('VarArg', [arg.variable]))
            elif kind == mypy.nodes.ARG_STAR2:
                extra.append(('DictVarArg', [arg.variable]))
        a = []  # type: List[Any]
        if args:
            a.append(('Args', args))
        if o.type:
            a.append(o.type)
        if o.is_generator:
            a.append('Generator')
        a.extend(extra)
        a.append(o.body)
        return a

    # Top-level structures

    def visit_mypy_file(self, o: 'mypy.nodes.MypyFile') -> str:
        # Skip implicit definitions.
        a = [o.defs]  # type: List[Any]
        if o.is_bom:
            a.insert(0, 'BOM')
        # Omit path to special file with name "main". This is used to simplify
        # test case descriptions; the file "main" is used by default in many
        # test cases.
        if o.path is not None and o.path != 'main':
            # Insert path. Normalize directory separators to / to unify test
            # case# output in all platforms.
            a.insert(0, o.path.replace(os.sep, '/'))
        if o.ignored_lines:
            a.append('IgnoredLines(%s)' % ', '.join(str(line)
                                                    for line in sorted(o.ignored_lines)))
        return self.dump(a, o)

    def visit_import(self, o: 'mypy.nodes.Import') -> str:
        a = []
        for id, as_id in o.ids:
            if as_id is not None:
                a.append('{} : {}'.format(id, as_id))
            else:
                a.append(id)
        return 'Import:{}({})'.format(o.line, ', '.join(a))

    def visit_import_from(self, o: 'mypy.nodes.ImportFrom') -> str:
        a = []
        for name, as_name in o.names:
            if as_name is not None:
                a.append('{} : {}'.format(name, as_name))
            else:
                a.append(name)
        return 'ImportFrom:{}({}, [{}])'.format(o.line, "." * o.relative + o.id, ', '.join(a))

    def visit_import_all(self, o: 'mypy.nodes.ImportAll') -> str:
        return 'ImportAll:{}({})'.format(o.line, "." * o.relative + o.id)

    # Definitions

    def visit_func_def(self, o: 'mypy.nodes.FuncDef') -> str:
        a = self.func_helper(o)
        a.insert(0, o.name())
        arg_kinds = {arg.kind for arg in o.arguments}
        if len(arg_kinds & {mypy.nodes.ARG_NAMED, mypy.nodes.ARG_NAMED_OPT}) > 0:
            a.insert(1, 'MaxPos({})'.format(o.max_pos))
        if o.is_abstract:
            a.insert(-1, 'Abstract')
        if o.is_static:
            a.insert(-1, 'Static')
        if o.is_class:
            a.insert(-1, 'Class')
        if o.is_property:
            a.insert(-1, 'Property')
        return self.dump(a, o)

    def visit_overloaded_func_def(self, o: 'mypy.nodes.OverloadedFuncDef') -> str:
        a = o.items[:]  # type: Any
        if o.type:
            a.insert(0, o.type)
        if o.impl:
            a.insert(0, o.impl)
        if o.is_static:
            a.insert(-1, 'Static')
        if o.is_class:
            a.insert(-1, 'Class')
        return self.dump(a, o)

    def visit_class_def(self, o: 'mypy.nodes.ClassDef') -> str:
        a = [o.name, o.defs.body]
        # Display base types unless they are implicitly just builtins.object
        # (in this case base_type_exprs is empty).
        if o.base_type_exprs:
            if o.info and o.info.bases:
                a.insert(1, ('BaseType', o.info.bases))
            else:
                a.insert(1, ('BaseTypeExpr', o.base_type_exprs))
        if o.type_vars:
            a.insert(1, ('TypeVars', o.type_vars))
        if o.metaclass:
            a.insert(1, 'Metaclass({})'.format(o.metaclass))
        if o.decorators:
            a.insert(1, ('Decorators', o.decorators))
        if o.info and o.info._promote:
            a.insert(1, 'Promote({})'.format(o.info._promote))
        if o.info and o.info.tuple_type:
            a.insert(1, ('TupleType', [o.info.tuple_type]))
        if o.info and o.info.fallback_to_any:
            a.insert(1, 'FallbackToAny')
        return self.dump(a, o)

    def visit_var(self, o: 'mypy.nodes.Var') -> str:
        lst = ''
        # Add :nil line number tag if no line number is specified to remain
        # compatible with old test case descriptions that assume this.
        if o.line < 0:
            lst = ':nil'
        return 'Var' + lst + '(' + o.name() + ')'

    def visit_global_decl(self, o: 'mypy.nodes.GlobalDecl') -> str:
        return self.dump([o.names], o)

    def visit_nonlocal_decl(self, o: 'mypy.nodes.NonlocalDecl') -> str:
        return self.dump([o.names], o)

    def visit_decorator(self, o: 'mypy.nodes.Decorator') -> str:
        return self.dump([o.var, o.decorators, o.func], o)

    # Statements

    def visit_block(self, o: 'mypy.nodes.Block') -> str:
        return self.dump(o.body, o)

    def visit_expression_stmt(self, o: 'mypy.nodes.ExpressionStmt') -> str:
        return self.dump([o.expr], o)

    def visit_assignment_stmt(self, o: 'mypy.nodes.AssignmentStmt') -> str:
        a = []  # type: List[Any]
        if len(o.lvalues) > 1:
            a = [('Lvalues', o.lvalues)]
        else:
            a = [o.lvalues[0]]
        a.append(o.rvalue)
        if o.type:
            a.append(o.type)
        return self.dump(a, o)

    def visit_operator_assignment_stmt(self, o: 'mypy.nodes.OperatorAssignmentStmt') -> str:
        return self.dump([o.op, o.lvalue, o.rvalue], o)

    def visit_while_stmt(self, o: 'mypy.nodes.WhileStmt') -> str:
        a = [o.expr, o.body]  # type: List[Any]
        if o.else_body:
            a.append(('Else', o.else_body.body))
        return self.dump(a, o)

    def visit_for_stmt(self, o: 'mypy.nodes.ForStmt') -> str:
        a = []  # type: List[Any]
        if o.is_async:
            a.append(('Async', ''))
        a.append(o.index)
        if o.index_type:
            a.append(o.index_type)
        a.extend([o.expr, o.body])
        if o.else_body:
            a.append(('Else', o.else_body.body))
        return self.dump(a, o)

    def visit_return_stmt(self, o: 'mypy.nodes.ReturnStmt') -> str:
        return self.dump([o.expr], o)

    def visit_if_stmt(self, o: 'mypy.nodes.IfStmt') -> str:
        a = []  # type: List[Any]
        for i in range(len(o.expr)):
            a.append(('If', [o.expr[i]]))
            a.append(('Then', o.body[i].body))

        if not o.else_body:
            return self.dump(a, o)
        else:
            return self.dump([a, ('Else', o.else_body.body)], o)

    def visit_break_stmt(self, o: 'mypy.nodes.BreakStmt') -> str:
        return self.dump([], o)

    def visit_continue_stmt(self, o: 'mypy.nodes.ContinueStmt') -> str:
        return self.dump([], o)

    def visit_pass_stmt(self, o: 'mypy.nodes.PassStmt') -> str:
        return self.dump([], o)

    def visit_raise_stmt(self, o: 'mypy.nodes.RaiseStmt') -> str:
        return self.dump([o.expr, o.from_expr], o)

    def visit_assert_stmt(self, o: 'mypy.nodes.AssertStmt') -> str:
        if o.msg is not None:
            return self.dump([o.expr, o.msg], o)
        else:
            return self.dump([o.expr], o)

    def visit_await_expr(self, o: 'mypy.nodes.AwaitExpr') -> str:
        return self.dump([o.expr], o)

    def visit_del_stmt(self, o: 'mypy.nodes.DelStmt') -> str:
        return self.dump([o.expr], o)

    def visit_try_stmt(self, o: 'mypy.nodes.TryStmt') -> str:
        a = [o.body]  # type: List[Any]

        for i in range(len(o.vars)):
            a.append(o.types[i])
            if o.vars[i]:
                a.append(o.vars[i])
            a.append(o.handlers[i])

        if o.else_body:
            a.append(('Else', o.else_body.body))
        if o.finally_body:
            a.append(('Finally', o.finally_body.body))

        return self.dump(a, o)

    def visit_with_stmt(self, o: 'mypy.nodes.WithStmt') -> str:
        a = []  # type: List[Any]
        if o.is_async:
            a.append(('Async', ''))
        for i in range(len(o.expr)):
            a.append(('Expr', [o.expr[i]]))
            if o.target[i]:
                a.append(('Target', [o.target[i]]))
        if o.target_type:
            a.append(o.target_type)
        return self.dump(a + [o.body], o)

    def visit_print_stmt(self, o: 'mypy.nodes.PrintStmt') -> str:
        a = o.args[:]  # type: List[Any]
        if o.target:
            a.append(('Target', [o.target]))
        if o.newline:
            a.append('Newline')
        return self.dump(a, o)

    def visit_exec_stmt(self, o: 'mypy.nodes.ExecStmt') -> str:
        return self.dump([o.expr, o.globals, o.locals], o)

    # Expressions

    # Simple expressions

    def visit_int_expr(self, o: 'mypy.nodes.IntExpr') -> str:
        return 'IntExpr({})'.format(o.value)

    def visit_str_expr(self, o: 'mypy.nodes.StrExpr') -> str:
        return 'StrExpr({})'.format(self.str_repr(o.value))

    def visit_bytes_expr(self, o: 'mypy.nodes.BytesExpr') -> str:
        return 'BytesExpr({})'.format(self.str_repr(o.value))

    def visit_unicode_expr(self, o: 'mypy.nodes.UnicodeExpr') -> str:
        return 'UnicodeExpr({})'.format(self.str_repr(o.value))

    def str_repr(self, s: str) -> str:
        s = re.sub(r'\\u[0-9a-fA-F]{4}', lambda m: '\\' + m.group(0), s)
        return re.sub('[^\\x20-\\x7e]',
                      lambda m: r'\u%.4x' % ord(m.group(0)), s)

    def visit_float_expr(self, o: 'mypy.nodes.FloatExpr') -> str:
        return 'FloatExpr({})'.format(o.value)

    def visit_complex_expr(self, o: 'mypy.nodes.ComplexExpr') -> str:
        return 'ComplexExpr({})'.format(o.value)

    def visit_ellipsis(self, o: 'mypy.nodes.EllipsisExpr') -> str:
        return 'Ellipsis'

    def visit_star_expr(self, o: 'mypy.nodes.StarExpr') -> str:
        return self.dump([o.expr], o)

    def visit_name_expr(self, o: 'mypy.nodes.NameExpr') -> str:
        pretty = self.pretty_name(o.name, o.kind, o.fullname, o.is_inferred_def, o.node)
        if isinstance(o.node, mypy.nodes.Var) and o.node.is_final:
            pretty += ' = {}'.format(o.node.final_value)
        return short_type(o) + '(' + pretty + ')'

    def pretty_name(self, name: str, kind: Optional[int], fullname: Optional[str],
                    is_inferred_def: bool, target_node: 'Optional[mypy.nodes.Node]' = None) -> str:
        n = name
        if is_inferred_def:
            n += '*'
        if target_node:
            id = self.format_id(target_node)
        else:
            id = ''
        if kind == mypy.nodes.GDEF or (fullname != name and
                                       fullname is not None):
            # Append fully qualified name for global references.
            n += ' [{}{}]'.format(fullname, id)
        elif kind == mypy.nodes.LDEF:
            # Add tag to signify a local reference.
            n += ' [l{}]'.format(id)
        elif kind == mypy.nodes.MDEF:
            # Add tag to signify a member reference.
            n += ' [m{}]'.format(id)
        else:
            n += id
        return n

    def visit_member_expr(self, o: 'mypy.nodes.MemberExpr') -> str:
        pretty = self.pretty_name(o.name, o.kind, o.fullname, o.is_inferred_def, o.node)
        return self.dump([o.expr, pretty], o)

    def visit_yield_expr(self, o: 'mypy.nodes.YieldExpr') -> str:
        return self.dump([o.expr], o)

    def visit_yield_from_expr(self, o: 'mypy.nodes.YieldFromExpr') -> str:
        if o.expr:
            return self.dump([o.expr.accept(self)], o)
        else:
            return self.dump([], o)

    def visit_call_expr(self, o: 'mypy.nodes.CallExpr') -> str:
        if o.analyzed:
            return o.analyzed.accept(self)
        args = []  # type: List[mypy.nodes.Expression]
        extra = []  # type: List[Union[str, Tuple[str, List[Any]]]]
        for i, kind in enumerate(o.arg_kinds):
            if kind in [mypy.nodes.ARG_POS, mypy.nodes.ARG_STAR]:
                args.append(o.args[i])
                if kind == mypy.nodes.ARG_STAR:
                    extra.append('VarArg')
            elif kind == mypy.nodes.ARG_NAMED:
                extra.append(('KwArgs', [o.arg_names[i], o.args[i]]))
            elif kind == mypy.nodes.ARG_STAR2:
                extra.append(('DictVarArg', [o.args[i]]))
            else:
                raise RuntimeError('unknown kind %d' % kind)
        a = [o.callee, ('Args', args)]  # type: List[Any]
        return self.dump(a + extra, o)

    def visit_op_expr(self, o: 'mypy.nodes.OpExpr') -> str:
        return self.dump([o.op, o.left, o.right], o)

    def visit_comparison_expr(self, o: 'mypy.nodes.ComparisonExpr') -> str:
        return self.dump([o.operators, o.operands], o)

    def visit_cast_expr(self, o: 'mypy.nodes.CastExpr') -> str:
        return self.dump([o.expr, o.type], o)

    def visit_reveal_expr(self, o: 'mypy.nodes.RevealExpr') -> str:
        if o.kind == mypy.nodes.REVEAL_TYPE:
            return self.dump([o.expr], o)
        else:
            # REVEAL_LOCALS
            return self.dump([o.local_nodes], o)

    def visit_unary_expr(self, o: 'mypy.nodes.UnaryExpr') -> str:
        return self.dump([o.op, o.expr], o)

    def visit_list_expr(self, o: 'mypy.nodes.ListExpr') -> str:
        return self.dump(o.items, o)

    def visit_dict_expr(self, o: 'mypy.nodes.DictExpr') -> str:
        return self.dump([[k, v] for k, v in o.items], o)

    def visit_set_expr(self, o: 'mypy.nodes.SetExpr') -> str:
        return self.dump(o.items, o)

    def visit_tuple_expr(self, o: 'mypy.nodes.TupleExpr') -> str:
        return self.dump(o.items, o)

    def visit_index_expr(self, o: 'mypy.nodes.IndexExpr') -> str:
        if o.analyzed:
            return o.analyzed.accept(self)
        return self.dump([o.base, o.index], o)

    def visit_super_expr(self, o: 'mypy.nodes.SuperExpr') -> str:
        return self.dump([o.name, o.call], o)

    def visit_type_application(self, o: 'mypy.nodes.TypeApplication') -> str:
        return self.dump([o.expr, ('Types', o.types)], o)

    def visit_type_var_expr(self, o: 'mypy.nodes.TypeVarExpr') -> str:
        import mypy.types
        a = []  # type: List[Any]
        if o.variance == mypy.nodes.COVARIANT:
            a += ['Variance(COVARIANT)']
        if o.variance == mypy.nodes.CONTRAVARIANT:
            a += ['Variance(CONTRAVARIANT)']
        if o.values:
            a += [('Values', o.values)]
        if not mypy.types.is_named_instance(o.upper_bound, 'builtins.object'):
            a += ['UpperBound({})'.format(o.upper_bound)]
        return self.dump(a, o)

    def visit_type_alias_expr(self, o: 'mypy.nodes.TypeAliasExpr') -> str:
        return 'TypeAliasExpr({})'.format(o.type)

    def visit_namedtuple_expr(self, o: 'mypy.nodes.NamedTupleExpr') -> str:
        return 'NamedTupleExpr:{}({}, {})'.format(o.line,
                                                  o.info.name(),
                                                  o.info.tuple_type)

    def visit_enum_call_expr(self, o: 'mypy.nodes.EnumCallExpr') -> str:
        return 'EnumCallExpr:{}({}, {})'.format(o.line, o.info.name(), o.items)

    def visit_typeddict_expr(self, o: 'mypy.nodes.TypedDictExpr') -> str:
        return 'TypedDictExpr:{}({})'.format(o.line,
                                             o.info.name())

    def visit__promote_expr(self, o: 'mypy.nodes.PromoteExpr') -> str:
        return 'PromoteExpr:{}({})'.format(o.line, o.type)

    def visit_newtype_expr(self, o: 'mypy.nodes.NewTypeExpr') -> str:
        return 'NewTypeExpr:{}({}, {})'.format(o.line, o.name,
                                               self.dump([o.old_type], o))

    def visit_lambda_expr(self, o: 'mypy.nodes.LambdaExpr') -> str:
        a = self.func_helper(o)
        return self.dump(a, o)

    def visit_generator_expr(self, o: 'mypy.nodes.GeneratorExpr') -> str:
        condlists = o.condlists if any(o.condlists) else None
        return self.dump([o.left_expr, o.indices, o.sequences, condlists], o)

    def visit_list_comprehension(self, o: 'mypy.nodes.ListComprehension') -> str:
        return self.dump([o.generator], o)

    def visit_set_comprehension(self, o: 'mypy.nodes.SetComprehension') -> str:
        return self.dump([o.generator], o)

    def visit_dictionary_comprehension(self, o: 'mypy.nodes.DictionaryComprehension') -> str:
        condlists = o.condlists if any(o.condlists) else None
        return self.dump([o.key, o.value, o.indices, o.sequences, condlists], o)

    def visit_conditional_expr(self, o: 'mypy.nodes.ConditionalExpr') -> str:
        return self.dump([('Condition', [o.cond]), o.if_expr, o.else_expr], o)

    def visit_slice_expr(self, o: 'mypy.nodes.SliceExpr') -> str:
        a = [o.begin_index, o.end_index, o.stride]  # type: List[Any]
        if not a[0]:
            a[0] = '<empty>'
        if not a[1]:
            a[1] = '<empty>'
        return self.dump(a, o)

    def visit_backquote_expr(self, o: 'mypy.nodes.BackquoteExpr') -> str:
        return self.dump([o.expr], o)

    def visit_temp_node(self, o: 'mypy.nodes.TempNode') -> str:
        return self.dump([o.type], o)


def dump_tagged(nodes: Sequence[object], tag: Optional[str], str_conv: 'StrConv') -> str:
    """Convert an array into a pretty-printed multiline string representation.

    The format is
      tag(
        item1..
        itemN)
    Individual items are formatted like this:
     - arrays are flattened
     - pairs (str, array) are converted recursively, so that str is the tag
     - other items are converted to strings and indented
    """
    from mypy.types import Type, TypeStrVisitor

    a = []  # type: List[str]
    if tag:
        a.append(tag + '(')
    for n in nodes:
        if isinstance(n, list):
            if n:
                a.append(dump_tagged(n, None, str_conv))
        elif isinstance(n, tuple):
            s = dump_tagged(n[1], n[0], str_conv)
            a.append(indent(s, 2))
        elif isinstance(n, mypy.nodes.Node):
            a.append(indent(n.accept(str_conv), 2))
        elif isinstance(n, Type):
            a.append(indent(n.accept(TypeStrVisitor(str_conv.id_mapper)), 2))
        elif n:
            a.append(indent(str(n), 2))
    if tag:
        a[-1] += ')'
    return '\n'.join(a)


def indent(s: str, n: int) -> str:
    """Indent all the lines in s (separated by newlines) by n spaces."""
    s = ' ' * n + s
    s = s.replace('\n', '\n' + ' ' * n)
    return s



"""Translate an Expression to a Type value."""

from mypy.nodes import (
    Expression, NameExpr, MemberExpr, IndexExpr, TupleExpr, IntExpr, FloatExpr, UnaryExpr,
    ComplexExpr, ListExpr, StrExpr, BytesExpr, UnicodeExpr, EllipsisExpr, CallExpr,
    get_member_expr_fullname
)
from mypy.fastparse import parse_type_string
from mypy.types import (
    Type, UnboundType, TypeList, EllipsisType, AnyType, Optional, CallableArgument, TypeOfAny,
    RawExpressionType,
)


class TypeTranslationError(Exception):
    """Exception raised when an expression is not valid as a type."""


def _extract_argument_name(expr: Expression) -> Optional[str]:
    if isinstance(expr, NameExpr) and expr.name == 'None':
        return None
    elif isinstance(expr, StrExpr):
        return expr.value
    elif isinstance(expr, UnicodeExpr):
        return expr.value
    else:
        raise TypeTranslationError()


def expr_to_unanalyzed_type(expr: Expression, _parent: Optional[Expression] = None) -> Type:
    """Translate an expression to the corresponding type.

    The result is not semantically analyzed. It can be UnboundType or TypeList.
    Raise TypeTranslationError if the expression cannot represent a type.
    """
    # The `parent` parameter is used in recursive calls to provide context for
    # understanding whether an CallableArgument is ok.
    name = None  # type: Optional[str]
    if isinstance(expr, NameExpr):
        name = expr.name
        if name == 'True':
            return RawExpressionType(True, 'builtins.bool', line=expr.line, column=expr.column)
        elif name == 'False':
            return RawExpressionType(False, 'builtins.bool', line=expr.line, column=expr.column)
        else:
            return UnboundType(name, line=expr.line, column=expr.column)
    elif isinstance(expr, MemberExpr):
        fullname = get_member_expr_fullname(expr)
        if fullname:
            return UnboundType(fullname, line=expr.line, column=expr.column)
        else:
            raise TypeTranslationError()
    elif isinstance(expr, IndexExpr):
        base = expr_to_unanalyzed_type(expr.base, expr)
        if isinstance(base, UnboundType):
            if base.args:
                raise TypeTranslationError()
            if isinstance(expr.index, TupleExpr):
                args = expr.index.items
            else:
                args = [expr.index]
            base.args = [expr_to_unanalyzed_type(arg, expr) for arg in args]
            if not base.args:
                base.empty_tuple_index = True
            return base
        else:
            raise TypeTranslationError()
    elif isinstance(expr, CallExpr) and isinstance(_parent, ListExpr):
        c = expr.callee
        names = []
        # Go through the dotted member expr chain to get the full arg
        # constructor name to look up
        while True:
            if isinstance(c, NameExpr):
                names.append(c.name)
                break
            elif isinstance(c, MemberExpr):
                names.append(c.name)
                c = c.expr
            else:
                raise TypeTranslationError()
        arg_const = '.'.join(reversed(names))

        # Go through the constructor args to get its name and type.
        name = None
        default_type = AnyType(TypeOfAny.unannotated)
        typ = default_type  # type: Type
        for i, arg in enumerate(expr.args):
            if expr.arg_names[i] is not None:
                if expr.arg_names[i] == "name":
                    if name is not None:
                        # Two names
                        raise TypeTranslationError()
                    name = _extract_argument_name(arg)
                    continue
                elif expr.arg_names[i] == "type":
                    if typ is not default_type:
                        # Two types
                        raise TypeTranslationError()
                    typ = expr_to_unanalyzed_type(arg, expr)
                    continue
                else:
                    raise TypeTranslationError()
            elif i == 0:
                typ = expr_to_unanalyzed_type(arg, expr)
            elif i == 1:
                name = _extract_argument_name(arg)
            else:
                raise TypeTranslationError()
        return CallableArgument(typ, name, arg_const, expr.line, expr.column)
    elif isinstance(expr, ListExpr):
        return TypeList([expr_to_unanalyzed_type(t, expr) for t in expr.items],
                        line=expr.line, column=expr.column)
    elif isinstance(expr, StrExpr):
        return parse_type_string(expr.value, 'builtins.str', expr.line, expr.column,
                                 assume_str_is_unicode=expr.from_python_3)
    elif isinstance(expr, BytesExpr):
        return parse_type_string(expr.value, 'builtins.bytes', expr.line, expr.column,
                                 assume_str_is_unicode=False)
    elif isinstance(expr, UnicodeExpr):
        return parse_type_string(expr.value, 'builtins.unicode', expr.line, expr.column,
                                 assume_str_is_unicode=True)
    elif isinstance(expr, UnaryExpr):
        typ = expr_to_unanalyzed_type(expr.expr)
        if isinstance(typ, RawExpressionType):
            if isinstance(typ.literal_value, int) and expr.op == '-':
                typ.literal_value *= -1
                return typ
        raise TypeTranslationError()
    elif isinstance(expr, IntExpr):
        return RawExpressionType(expr.value, 'builtins.int', line=expr.line, column=expr.column)
    elif isinstance(expr, FloatExpr):
        # Floats are not valid parameters for RawExpressionType , so we just
        # pass in 'None' for now. We'll report the appropriate error at a later stage.
        return RawExpressionType(None, 'builtins.float', line=expr.line, column=expr.column)
    elif isinstance(expr, ComplexExpr):
        # Same thing as above with complex numbers.
        return RawExpressionType(None, 'builtins.complex', line=expr.line, column=expr.column)
    elif isinstance(expr, EllipsisExpr):
        return EllipsisType(expr.line)
    else:
        raise TypeTranslationError()



from typing import Any

COLLECTION_ACTIONS = ...  # type: Any
MEMBER_ACTIONS = ...  # type: Any

def strip_slashes(name): ...

class SubMapperParent:
    def submapper(self, **kargs): ...
    def collection(self, collection_name, resource_name, path_prefix=..., member_prefix=..., controller=...,
                   collection_actions=..., member_actions=..., member_options=..., **kwargs): ...

class SubMapper(SubMapperParent):
    kwargs = ...  # type: Any
    obj = ...  # type: Any
    collection_name = ...  # type: Any
    member = ...  # type: Any
    resource_name = ...  # type: Any
    formatted = ...  # type: Any
    def __init__(self, obj, resource_name=..., collection_name=..., actions=..., formatted=..., **kwargs) -> None: ...
    def connect(self, *args, **kwargs): ...
    def link(self, rel=..., name=..., action=..., method=..., formatted=..., **kwargs): ...
    def new(self, **kwargs): ...
    def edit(self, **kwargs): ...
    def action(self, name=..., action=..., method=..., formatted=..., **kwargs): ...
    def index(self, name=..., **kwargs): ...
    def show(self, name=..., **kwargs): ...
    def create(self, **kwargs): ...
    def update(self, **kwargs): ...
    def delete(self, **kwargs): ...
    def add_actions(self, actions): ...
    def __enter__(self): ...
    def __exit__(self, type, value, tb): ...

class Mapper(SubMapperParent):
    matchlist = ...  # type: Any
    maxkeys = ...  # type: Any
    minkeys = ...  # type: Any
    urlcache = ...  # type: Any
    prefix = ...  # type: Any
    req_data = ...  # type: Any
    directory = ...  # type: Any
    always_scan = ...  # type: Any
    controller_scan = ...  # type: Any
    debug = ...  # type: Any
    append_slash = ...  # type: Any
    sub_domains = ...  # type: Any
    sub_domains_ignore = ...  # type: Any
    domain_match = ...  # type: Any
    explicit = ...  # type: Any
    encoding = ...  # type: Any
    decode_errors = ...  # type: Any
    hardcode_names = ...  # type: Any
    minimization = ...  # type: Any
    create_regs_lock = ...  # type: Any
    def __init__(self, controller_scan=..., directory=..., always_scan=..., register=..., explicit=...) -> None: ...
    environ = ...  # type: Any
    def extend(self, routes, path_prefix=...): ...
    def make_route(self, *args, **kargs): ...
    def connect(self, *args, **kargs): ...
    def create_regs(self, *args, **kwargs): ...
    def match(self, url=..., environ=...): ...
    def routematch(self, url=..., environ=...): ...
    obj = ...  # type: Any
    def generate(self, *args, **kargs): ...
    def resource(self, member_name, collection_name, **kwargs): ...
    def redirect(self, match_path, destination_path, *args, **kwargs): ...
